unit fOptions;

interface

uses
  Windows, Sysutils, Classes, Controls, Forms, graphics,
  Dialogs, ExtDlgs, StdCtrls, Buttons, ExtCtrls, ComCtrls, VirtualTrees;

type
  TPrefForm = class(TForm)
    BtnOk: TButton;
    BtnCancel: TButton;
    BtnApply: TButton;
    PgeOptions: TPageControl;
    TabGeneral: TTabSheet;
    GrpGeneral: TGroupBox;
    ChkRunDlg: TCheckBox;
    ChkMinimise: TCheckBox;
    ChkAutoDel: TCheckBox;
    GrpMedia: TGroupBox;
    RadMatchAll: TRadioButton;
    RadStartWith: TRadioButton;
    RadInclude: TRadioButton;
    ChkSearchROMdir: TCheckBox;
    ChkClearFilter: TCheckBox;
    TabRomAppearance: TTabSheet;
    GrpRomsOptions: TGroupBox;
    ChkRomGridlines: TCheckBox;
    ChkListImages: TCheckBox;
    ChkROMHotTrack: TCheckBox;
    ChkIncVersion: TCheckBox;
    GrpRomColours: TGroupBox;
    GrpRomFont: TGroupBox;
    RomSt: TLabel;
    RomSi: TLabel;
    RomN: TLabel;
    RomC: TShape;
    BtnRomFont: TButton;
    CmbRTrack: TColorBox;
    CmbRSelUnFoc: TColorBox;
    CmbRSel: TColorBox;
    CmbRBKCol: TColorBox;
    BtnRomDefaults: TButton;
    TabDirSettings: TTabSheet;
    GrpDirColour: TGroupBox;
    GrpDirFont: TGroupBox;
    DirSt: TLabel;
    DirSi: TLabel;
    DirN: TLabel;
    DirC: TShape;
    Button1: TButton;
    CmbDTrack: TColorBox;
    CmbDSelUnFoc: TColorBox;
    CmbDSel: TColorBox;
    CmbDBKCol: TColorBox;
    BtnDirDefaults: TButton;
    GrpDirOptions: TGroupBox;
    ChkDirGridlines: TCheckBox;
    ChkDirHotTrack: TCheckBox;
    ChkDirRowSel: TCheckBox;
    ChkCol: TCheckBox;
    ChkRomRowSelect: TCheckBox;
    GrpDefaultIni: TPanel;
    BtnDelReg: TButton;
    GrpReset: TPanel;
    LblResetTots: TLabel;
    BtnReset: TButton;
    GrpSystem: TPanel;
    BtnSystem: TButton;
    GrpLanguage: TPanel;
    BtnEditLang: TButton;
    GrpTypes: TPanel;
    BtnEditTypes: TButton;
    ChkCloneSearch: TCheckBox;
    ChkRomHints: TCheckBox;
    ChkDirHints: TCheckBox;
    GrpColView: TGroupBox;
    VTShowCols: TVirtualStringTree;
    ChkRememberROM: TCheckBox;
    ChkTempDir: TCheckBox;
    ChkOldMini: TCheckBox;
    TxtDirBK: TEdit;
    lblPath: TLabel;
    chkDirBK: TCheckBox;
    BtnDirBKOpen: TButton;
    procedure BtnOKClick(Sender: TObject);
    procedure BtnApplyClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BtnSystemClick(Sender: TObject);
    procedure BtnResetClick(Sender: TObject);
    procedure BtnDelRegClick(Sender: TObject);
    procedure BtnEditLangClick(Sender: TObject);
    procedure BtnEditTypesClick(Sender: TObject);
    procedure BtnRomDefaultsClick(Sender: TObject);
    procedure BtnDirDefaultsClick(Sender: TObject);
    procedure BtnRomFontClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure VTShowColsGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure BtnDirBKOpenClick(Sender: TObject);
    procedure chkDirBKClick(Sender: TObject);
  private
    procedure FontChange(Own: TComponent; Name, Size, Style: TLabel;
      Colour: TShape);
    Procedure ShowSettings();
    { Private declarations }
  public
    { Public declarations }
    Procedure SaveSettings();

  end;

implementation

uses fMain, fTxtEditor, uJUtilities, uRomList, inifiles, uRomUtils, uQPConst;

{$R *.DFM}
{-----------------------------------------------------------------------------}

procedure TPrefForm.FontChange(Own : TComponent; Name, Size, Style : TLabel; Colour : TShape);
var
  FontD : TFontDialog;
begin
  FontD := TFontDialog.Create(Self);
  try
    if Name.Caption <> '' then
      FontD.Font.Name := name.Caption
    else
      FontD.Font.Name := 'Tahoma';

    if Size.Caption <> '' then
      FontD.Font.Size := StrToInt(Size.Caption)
    else
      FontD.Font.Size := 8;

    FontD.Font.Color := Colour.Brush.Color;

    if pos('Bold', style.caption) <> 0 then
      FontD.Font.Style := FontD.font.Style + [fsbold];

    if pos('Italic', style.caption) <> 0 then
      FontD.Font.Style := FontD.font.Style + [fsItalic];

    if pos('Underline', style.caption) <> 0 then
      FontD.Font.Style := FontD.font.Style + [fsUnderLine];

    if pos('Strikethrough', style.caption) <> 0 then
      FontD.Font.Style := FontD.font.Style + [fsstrikeout];

    If FontD.Execute then
    Begin
      name.Caption := FontD.Font.Name;
      size.Caption := IntToStr(FontD.Font.Size);
      Colour.Brush.Color := FontD.Font.Color;

      Style.Caption := '';


      if FontD.Font.Style >= [fsBold] then
        style.Caption := Style.Caption + 'Bold ';

      if FontD.Font.Style >= [fsItalic] = true then
        Style.Caption := Style.Caption + 'Italic ';

      if FontD.Font.Style >= [fsUnderLine] = true then
        Style.Caption := Style.Caption + 'Underline ';

      if FontD.Font.Style >= [fsstrikeout] = true then
        Style.Caption := Style.Caption + 'Strikethrough ';
    end;

  finally
    FreeAndNil(FontD);
  end;
end;

{-----------------------------------------------------------------------------}

Procedure TPrefForm.SaveSettings();
var
  aNode : PVirtualNode;
  Ini : TMemIniFile;
begin

    //General Options
    With MainFrm do
    begin
      Settings.AutoDelRoms := ChkAutoDel.Checked;
      Settings.MiniOnRun := ChkMinimise.Checked;
      Settings.UseOldMini := ChkOldMini.Checked;
      Settings.UseRunDlg := ChkRunDlg.Checked;
      Settings.ClearFilter := ChkClearFilter.Checked;
      Settings.RememberLast := ChkRememberROM.Checked;
      Settings.UseTempDir := ChkTempDir.Checked;

      //Media Path options
      Settings.SearchROMDir := ChkSearchROMDir.Checked;
      Settings.ParentsForClones := ChkCloneSearch.Checked;

      If RadMatchAll.Checked then
        Settings.SearchType := 1
      else
        if RadStartWith.Checked then
          Settings.SearchType := 2
        else
          if RadInclude.Checked then
            Settings.SearchType := 3;


      //ROM list options
      
      Settings.SaveColumnSize := ChkCol.Checked;
      Settings.UseImagesForROMs := ChkListImages.Checked;
      Settings.AddVersion := ChkIncVersion.Checked;
      VTRoms.ShowHint := ChkROMHints.Checked;

      //VTROM treeoptions.
      with MainFrm.VTRoms.TreeOptions do
      begin

        case ChkRomRowSelect.Checked of
          false: SelectionOptions := (SelectionOptions - [toFullRowSelect]);
          true: SelectionOptions := (SelectionOptions + [toFullRowSelect]);
        end;

        case chkROMGridlines.Checked of
          false: PaintOptions := (PaintOptions-[toShowHorzGridLines, toShowVertGridLines]);
          true: PaintOptions := (PaintOptions+[toShowHorzGridLines, toShowVertGridLines]);
        end;

        //Load the HotTrack on VTRoms option
        case ChkROMHotTrack.Checked of
          false: PaintOptions := PaintOptions-[TohotTrack];
          true: PaintOptions := PaintOptions+[TohotTrack];
        end;

      end;  //end of with MainFrm.VTRoms.treeoptions code


      //this is the only time we need to DIRECTLY access the ini file.
      //simply because I dont want to store these in an Array
      Ini := TMemIniFile.Create(Settings.SettingsFile);
      try
        //save the default columns
        Ini.EraseSection('ColView');

        aNode := VTShowCols.GetFirst;
        while aNode <> nil do
        begin
          If VTShowCols.CheckState[aNode] = csCheckedNormal then
            Ini.WriteBool('ColView', IntToStr(aNode.Index), True)
          else
            Ini.WriteBool('ColView', IntToStr(aNode.Index), false);
          aNode := VTShowCols.GetNext(aNode);
        end;

        If Settings.AllowWrite then
          Ini.UpdateFile;
      Finally
        FreeAndNil(Ini);
      end;

      //Rom List Colours
      VTROMs.Color := CmbRBKCol.Selected;
      VTROMs.Colors.FocusedSelectionColor := CmbRSel.Selected;
      VTROMs.Colors.FocusedSelectionBorderColor := CmbRSel.Selected;
      VTROMs.Colors.UnfocusedSelectionColor := CmbRSelUnFoc.Selected;
      VTROMs.Colors.UnfocusedSelectionBorderColor := CmbRSelUnFoc.Selected;
      VTROMS.Colors.HotColor := CmbRTrack.Selected;

      VTROMs.Font.Name := ROMN.Caption;
      VTROMs.Font.Color := ROMC.Brush.Color;
      VTROMs.Font.Size := StrToInt(RomSi.Caption);

      VTROMS.Font.Style := [];
      if pos('Bold', RomST.Caption) <> 0 then
        VTROMS.font.Style := VTROMS.font.Style + [fsbold];

      if pos('Italic', RomST.Caption) <> 0 then
        VTROMS.font.Style := VTROMS.font.Style + [fsItalic];

      if pos('Underline', RomST.Caption) <> 0 then
        VTROMS.font.Style := VTROMS.font.Style + [fsUnderLine];

      if pos('Strikethrough', RomST.Caption) <> 0 then
        VTROMS.font.Style := VTROMS.font.Style + [fsstrikeout];

      // Directory List Options
      
      VTDir.ShowHint := ChkDirHints.Checked;

      Settings.UseBGonDir := ChkDirBK.Checked;
      Settings.DirBGImage := TxtDirBK.Text;

      with MainFrm.VTDir.TreeOptions do
      begin

        case chkDirRowSel.Checked of
          false: SelectionOptions := (SelectionOptions - [toFullRowSelect]);
          true: SelectionOptions := (SelectionOptions + [toFullRowSelect]);
        end;

        case chkDirGridlines.Checked of
          false: PaintOptions := (PaintOptions-[toShowHorzGridLines, toShowVertGridLines]);
          true: PaintOptions := (PaintOptions+[toShowHorzGridLines, toShowVertGridLines]);
        end;

        //Load the HotTrack on VTRoms option
        case ChkDirHotTrack.Checked of
          false: PaintOptions := PaintOptions-[TohotTrack];
          true: PaintOptions := PaintOptions+[TohotTrack];
        end;

      end;  //end of with MainFrm.VTdir.treeoptions code

      VTdir.Color := CmbDBKCol.Selected;
      VTdir.Colors.FocusedSelectionColor := CmbDSel.Selected;
      VTdir.Colors.FocusedSelectionBorderColor := CmbDSel.Selected;
      VTdir.Colors.UnfocusedSelectionColor := CmbDSelUnFoc.Selected;
      VTdir.Colors.UnfocusedSelectionBorderColor := CmbDSelUnFoc.Selected;
      VTdir.Colors.HotColor := CmbDTrack.Selected;

      VTdir.Font.Name := DirN.Caption;
      VTdir.Font.Color := DirC.Brush.Color;
      VTdir.Font.Size := StrToInt(DirSi.Caption);

      VTdir.Font.Style := [];
      if pos('Bold', DirSt.Caption) <> 0 then
        VTdir.font.Style := VTdir.font.Style + [fsbold];

      if pos('Italic', DirSt.Caption) <> 0 then
        VTdir.font.Style := VTdir.font.Style + [fsItalic];

      if pos('Underline', DirSt.Caption) <> 0 then
        VTdir.font.Style := VTdir.font.Style + [fsUnderLine];

      if pos('Strikethrough', DirSt.Caption) <> 0 then
        VTdir.font.Style := VTdir.font.Style + [fsstrikeout];

      //now call the SAVE method for the settings variable.
      Settings.SaveAllSettings();
    end;
end;

{-----------------------------------------------------------------------------}

Procedure TPrefForm.ShowSettings();
var
  Ini : TMemInifile;
  Node : PVirtualNode;
begin
  Ini := TMemInifile.create(MainFrm.Settings.SettingsFile);
  try

    //set up the columns treeview
    VTShowCols.RootNodeCount := MainFrm.VTRoms.Header.Columns.Count;

    Node := VTShowCols.GetFirst;
    while Node <> nil do
    begin
      VTShowCols.CheckType[Node] := ctCheckBox;
      If Ini.ReadBool('ColView', IntToStr(Node.Index), true) then
        VTShowCols.CheckState[Node] := csCheckedNormal;
      Node := Node.NextSibling;
    end;

  finally
    FreeAndNil(Ini);
  end;

  with MainFrm do
  begin
    //general options
    ChkAutoDel.Checked := Settings.AutoDelRoms;
    ChkMinimise.Checked := Settings.MiniOnRun;
    ChkOldMini.Checked := Settings.UseOldMini;
    ChkRunDlg.Checked := Settings.UseRunDlg;
    ChkClearFilter.Checked := Settings.ClearFilter;
    ChkRememberROM.Checked := Settings.RememberLast;
    ChkTempDir.Checked := Settings.UseTempDir;

    //Media Search options
    ChkSearchROMdir.Checked := Settings.SearchROMDir;
    ChkCloneSearch.Checked := Settings.ParentsForClones;
    Case Settings.SearchType of
      1: RadMatchAll.Checked := True;
      2: RadStartWith.Checked := True;
      3: RadInclude.Checked := True;
    end;

    // ROM list options
    ChkListImages.Checked := Settings.UseImagesForROMs;
    ChkIncVersion.Checked := Settings.AddVersion;
    ChkCol.Checked := Settings.SaveColumnSize;
    ChkROMhints.Checked := VTROMs.ShowHint;
    ChkRomRowSelect.Checked := (toFullRowSelect in VTRoms.TreeOptions.SelectionOptions);
    ChkROMHotTrack.Checked := (toHotTrack in VTRoms.TreeOptions.PaintOptions);
    ChkROMGridlines.Checked := (toShowHorzGridLines in VTRoms.TreeOptions.PaintOptions);

    //Colour Setup
    CmbRBKCol.Selected := VTROMs.Color;
    CmbRSel.Selected := VTROMs.Colors.FocusedSelectionColor;
    CmbRSelUnFoc.Selected := VTROMs.Colors.UnfocusedSelectionColor;
    CmbRTrack.Selected := VTROMs.Colors.HotColor;

    //font setup
    RomN.Caption := VTROMs.Font.Name;
    RomC.Brush.Color := VTROMs.Font.Color;
    RomSi.Caption := IntToStr(VTROMs.Font.Size);

    RomSt.caption := '';
    if (fsBold in VTROMs.Font.Style) then
      RomSt.Caption := RomSt.Caption + 'Bold ';

    if (fsItalic in VTROMs.Font.Style) then
      RomSt.Caption := RomSt.Caption + 'Italic ';

    if (fsUnderLine in VTROMs.Font.Style) then
      RomSt.Caption := RomSt.Caption + 'Underline ';

    if (fsStrikeOut in VTROMs.Font.Style) then
      RomSt.Caption := RomSt.Caption + 'Strikethrough ';
    // end of ROM list setup code

    // Directory list options setup
    ChkDirBK.Checked := Settings.UseBGonDir;
    ChkDirBK.OnClick(Self);
    TxtDirBK.Text := Settings.DirBGImage;

    ChkDirRowSel.Checked := (toFullRowSelect in VTdir.TreeOptions.SelectionOptions);
    ChkDirHotTrack.Checked := (toHotTrack in VTdir.TreeOptions.PaintOptions);
    ChkDirGridlines.Checked := (toShowHorzGridLines in VTdir.TreeOptions.PaintOptions);
    ChkDirHints.Checked := VTdir.ShowHint;

    //Colour Setup
    CmbDBKCol.Selected := VTdir.Color;
    CmbDSel.Selected := VTdir.Colors.FocusedSelectionColor;
    CmbDSelUnFoc.Selected := VTdir.Colors.UnfocusedSelectionColor;
    CmbDTrack.Selected := VTdir.Colors.HotColor;

    //font setup
    DirN.Caption := VTdir.Font.Name;
    DirC.Brush.Color := VTdir.Font.Color;
    DirSi.Caption := IntToStr(VTdir.Font.Size);

    DirSt.caption := '';
    if (fsBold in VTdir.Font.Style) then
      DirSt.Caption := DirSt.Caption + 'Bold ';

    if (fsItalic in VTdir.Font.Style) then
      DirSt.Caption := DirSt.Caption + 'Italic ';

    if (fsUnderLine in VTdir.Font.Style) then
      DirSt.Caption := DirSt.Caption + 'Underline ';

    if (fsStrikeOut in VTdir.Font.Style) then
      DirSt.Caption := DirSt.Caption + 'Strikethrough ';
  end;

end;

{-----------------------------------------------------------------------------}

procedure TPrefForm.BtnOKClick(Sender: TObject);
begin
  SaveSettings();
end;

{-----------------------------------------------------------------------------}

procedure TPrefForm.BtnApplyClick(Sender: TObject);
begin
  SaveSettings();
  ShowSettings();
end;

{-----------------------------------------------------------------------------}

procedure TPrefForm.FormShow(Sender: TObject);
begin
  PgeOptions.ActivePageIndex := 0;
  showsettings();
end;

{-----------------------------------------------------------------------------}

procedure TPrefForm.BtnSystemClick(Sender: TObject);
var
  edit : TeditCfgForm;
begin
  edit := TEditCfgForm.create(Self);
  try
    if fileexists(MainFrm.Settings.SettingsDir+'systems.dat') then
    begin
      edit.load(MainFrm.Settings.SettingsDir+'systems.dat');
      edit.ShowModal;
    end
    else
      MessageDlg(QP_FILE_NOT_EXIST, mtError, [mbOK], 0);
  finally
    edit.Release;
  end;
end;

{-----------------------------------------------------------------------------}

procedure TPrefForm.BtnResetClick(Sender: TObject);
var
  i, e : integer;
  ROMFiles : TSTringList;
  TmpList : TQPROMList;
begin
  ROMFiles := TStringList.Create;
  Try
    uROMUtils.GetAllROMFiles(ROMFiles);

    TmpList := TQPROMlist.Create(MainFrm.Settings.AllowWrite);
    try
      for i := 0 to ROMFiles.Count-1 do
      begin

        TmpList.LoadFromFile(ROMFiles[i]);
        for e := 0 to TmpList.Count-1 do
          TmpList[e].NumPlay := 0;

        TmpList.SaveToFile();

      end;  //end of FOR block

      TmpList.Clear;
      
    finally
      FreeAndNil(TmpList);
    end;
                      
  Finally
    FreeAndNil(ROMFiles);
  end;
end;

{-----------------------------------------------------------------------------}

procedure TPrefForm.BtnDelRegClick(Sender: TObject);
begin
  MainFrm.Settings.CreateDefaultIni(true, MainFrm.Settings.SettingsFile);
end;

{-----------------------------------------------------------------------------}

procedure TPrefForm.BtnEditLangClick(Sender: TObject);
var
  edit : TeditCfgForm;
begin
  edit := TEditCfgForm.create(Self);
  try
    if fileexists(MainFrm.Settings.SettingsDir+'language.txt') then
    begin
      edit.load(MainFrm.Settings.SettingsDir+'language.txt');
      edit.ShowModal;
    end
    else
      MessageDlg(QP_FILE_NOT_EXIST, mtError, [mbOK], 0);
  finally
    edit.Release;
  end;
end;

procedure TPrefForm.BtnEditTypesClick(Sender: TObject);
var
  edit : TeditCfgForm;
begin
  edit := TEditCfgForm.create(Self);
  try
    if fileexists(MainFrm.Settings.SettingsDir+'type.txt') then
    begin
      edit.load(MainFrm.Settings.SettingsDir+'type.txt');
      edit.ShowModal;
    end
    else
      MessageDlg(QP_FILE_NOT_EXIST, mtError, [mbOK], 0);
  finally
    edit.Release;
  end;
end;

procedure TPrefForm.BtnRomDefaultsClick(Sender: TObject);
begin
  CmbRBkCol.Selected := clWindow;
  CmbRSel.Selected := clSkyblue;
  CmbRSelUnFoc.Selected := clSilver;
  CmbRTrack.Selected := clBlue;
  RomC.Brush.Color := clBlack;
  RomN.Caption := 'Tahoma';
  RomSi.Caption := IntToStr(8);
  RomSt.Caption := '';
end;

procedure TPrefForm.BtnDirDefaultsClick(Sender: TObject);
begin
  CmbDBkCol.Selected := clWindow;
  CmbDSel.Selected := clSkyblue;
  CmbDSelUnFoc.Selected := clSilver;
  CmbDTrack.Selected := clBlue;
  DirC.Brush.Color := clBlack;
  DirN.Caption := 'Tahoma';
  DirSi.Caption := IntToStr(8);
  DirSt.Caption := '';
end;

procedure TPrefForm.BtnRomFontClick(Sender: TObject);
begin
  FontChange(Self, RomN, RomSi, RomSt, RomC);
end;

procedure TPrefForm.Button1Click(Sender: TObject);
begin
  FontChange(Self, DirN, DirSi, DirSt, DirC);
end;

procedure TPrefForm.VTShowColsGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
begin
  CellText := MainFrm.VTRoms.Header.Columns.Items[Node.Index].Text;
end;

procedure TPrefForm.BtnDirBKOpenClick(Sender: TObject);
var
  PicDlg : TOpenPictureDialog;
begin

  PicDlg := TOpenPictureDialog.Create(self);
  try

    if DirectoryExists(MainFrm.Settings.QPDirectory + 'images') then
      PicDlg.InitialDir := MainFrm.Settings.QPDirectory + 'images';

    PicDlg.Filter := 'GIF Images (*.gif)|*.gif|PNG Images (*.png)|*.png|Bitmap Images|*.bmp|All Files|*.*';
    if PicDlg.Execute then
      if (PicDlg.FileName <> '') AND (fileexists(PicDlg.FileName)) then
        TxtDirBK.Text := PicDlg.FileName;

  finally
    FreeAndNil(PicDlg);
  end;
end;

procedure TPrefForm.chkDirBKClick(Sender: TObject);
begin
  lblPath.Enabled := ChkDirBK.Checked;
  TxtDirBK.Enabled := ChkDirBK.Checked;
  BtnDirBKOpen.Enabled := ChkDirBK.Checked;
end;

end.
