unit fRomProperties;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, VirtualTrees, Graphics,  
  Dialogs, StdCtrls, Buttons, Grids, ValEdit, uRomList, uRom, ComCtrls, Contnrs;

type
  TFrmRomProp = class(TForm)
    BtnOK: TButton;
    BtnCancel: TButton;
    pgProp: TPageControl;
    TabIPS: TTabSheet;
    TabFile: TTabSheet;
    LstContent: TListBox;
    lblValid: TLabel;
    lblIsZip: TLabel;
    lblFileSize: TLabel;
    LblExist: TLabel;
    Label1: TLabel;
    IPSeditor: TValueListEditor;
    BtnIPSdel: TButton;
    BtnIPSinvalid: TButton;
    BtnPrevious: TButton;
    BtnNext: TButton;
    BtnIPSAdd: TButton;
    TabProperties: TTabSheet;
    VTData: TVirtualStringTree;
    procedure FormShow(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnIPSdelClick(Sender: TObject);
    procedure BtnIPSinvalidClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BtnIPSAddClick(Sender: TObject);
    procedure BtnNextClick(Sender: TObject);
    procedure BtnPreviousClick(Sender: TObject);
    procedure VTDataEditing(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; var Allowed: Boolean);
    procedure VTDataChange(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure VTDataGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure VTDataCreateEditor(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; out EditLink: IVTEditLink);
    procedure VTDataNewText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; NewText: WideString);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    _ROM : TQPROM;
    _ROMref : TQProm;
    _Changed : Boolean;
    Procedure Init;
    procedure EditROM();
    procedure Move(GoForward : Boolean);
    procedure SetROM(value : TQPROM);
  public
    { Public declarations }
    Selected : TObjectList;
    Multi : Boolean;
    cNode : PVirtualNode;

    Property ROM : TQPROM read _ROMref write SetROM;
  end;

implementation

uses JCLFileUtils, fMain, uJZip, uJUtilities, uEmuList, fIPS, uQPVTEditor, uQPConst;

{$R *.dfm}

{ TFrmRomProp }

procedure TFrmRomProp.SetROM(value : TQPROM);
begin
  _ROMref := value;
  _ROM.Assign(_ROMref);
  _Changed := False;
end;

procedure TFrmRomProp.EditROM();
var
  i, iSize : Integer;
begin

  //if the tree is still editing, stop it
  if VTData.IsEditing then
    VTData.EndEditNode; 

  if (FileExists(_ROM.Path) = false) or (DirectoryExists(ExtractFilePath(_ROM.Path)) = false) then
    raise EJException.Create(QP_ROM_NOT_FOUND);

  //edit the IPS values
  with _ROM do
  Begin
    isize := 0;
    Setlength(IPS, isize);
    For I := 1 to IPSeditor.RowCount-1 do
    begin

      If (IPSEditor.Cells[0, I] <> '') AND (IPSEditor.Cells[1,I] <> '') then
      Begin
        inc(Isize);
        SetLength(IPS, Isize);
        IPS[Isize-1].Index := Isize-1;
        IPS[Isize-1].Name := IPSEditor.Cells[0, I];
        IPS[Isize-1].Path := IPSeditor.Cells[1,I];
      End;

    End;

  end;//end of WITH block

  //finally assign the internal ROM data to the original one.
  _ROMref.Assign(_ROM);
  
end;

procedure TFrmRomProp.Init;
Var
  I : Integer;
  TmpList : TStrings;
  aNode : PVirtualNode;
begin
  with MainFrm do
  Begin  
  //Start filling the data for the roms.

  If Multi then
  begin //start of multiple selected roms

    //first of all disable the next and back buttons
    BtnNext.Enabled := False;
    BtnPrevious.Enabled := False;

    //now set up our internal _ROM object to 'unchanged' values
    _ROM.Emulator := QP_UNCHANGED;
    _ROM.Language := QP_UNCHANGED;
    _ROM.Rating := QP_UNCHANGED;
    _ROM.GameType := QP_UNCHANGED;
    _ROM.Multiplayer := QP_UNCHANGED;
    _ROM.Company := QP_UNCHANGED;

    //now hide the nodes we arent interested in.
    aNode := VTData.GetFirst;
    While aNode <> nil do
    begin
      if aNode.Index in [0, 1, 4, 9, 10, 11] then
        VTData.IsVisible[aNode] := False;
      aNode := VTData.GetNext(aNode);
    end;
    
  end //end of multiple selected roms

  else

  begin //start of single selected rom

    //IPS data
    with _ROM do
    for I := 0 to Length(IPS)-1 do
    Begin
      IPSeditor.InsertRow(IPS[i].Name, IPS[i].Path, true);
    End;

    // file values
    if FileExists(_ROM.path) then
    begin
      LblExist.Caption := QP_YES;

      lblFileSize.Caption := IntToStr(GetSizeOfFile(_ROM.path))+' bytes';

      if lowercase(ExtractFileExt(_ROM.path)) = '.zip' then
      Begin
        LblIsZip.Caption := QP_YES;

        If VerifyZip(_ROM.path) then
        begin
          LblValid.Caption := QP_YES;
          With MainFrm do
          begin

            TmpList := TStringList.Create;
            try
              uJZip.GetZipContents(TmpList, _ROM.Path);
              LstContent.Items.AddStrings(TmpList);
            Finally
              FreeAndNil(TmpList);
            end;
            {if Unzip.Count > 0 then
              for I := 0 to Unzip.Count-1 do
                LstContent.Items.Add(Unzip.DirEntry[i].FileName);  }

          end;  //end of WITH MainFrm
        end //end of VerifyZip
        else
          LblValid.Caption := QP_NO;

      end; //end of IS ZIP file

    end;

  end; //end of single selected rom
  end; //end of WITH block
end;

procedure TFrmRomProp.Move(GoForward : Boolean);
var
  DlgResult : Integer;
  aNode : PVirtualNode;
begin

  if VTData.IsEditing then
    VTData.EndEditNode;

  if _Changed then
  begin
    DlgResult := MessageDlg(QP_SAVE_CHANGES, mtConfirmation, [mbYes,mbNo,mbCancel], 0);
    if DlgResult = mrCancel then
    begin
      //Dont do anything, just result and exit;
      exit;
    end
    else if DlgResult = mrYes then
    begin
      EditROM();
      MainFrm.RomList.SaveToFile();
    end;

  end;

  //Move to the next ROM
  if GoForward then
    aNode := MainFrm.VTRoms.GetNext(cNode)
  else
    aNode := MainFrm.VTRoms.GetPrevious(cNode);

  if aNode <> nil then
  begin
    cNode := aNode;
    Self.ROM := MainFrm.GetROM(aNode);
    Init();
    VTData.Invalidate;
  end
  else
    //its at the end of the list.
    Raise EJException.Create(QP_ROMPROP_TOP);

end;

procedure TFrmRomProp.BtnOKClick(Sender: TObject);
Var
  I : Integer;
begin
  With MainFrm do
  Begin

  if VTData.IsEditing then
      VTData.EndEditNode;

  if Multi then
  Begin

    For I := 0 to Selected.Count-1 do
    Begin
        If _ROM.Emulator <> QP_UNCHANGED then
          TQPROM(Selected[I]).emulator := _ROM.Emulator;
        If _ROM.GameType <> QP_UNCHANGED then
          TQPROM(Selected[I]).GameType := _ROM.GameType;
        If _ROM.Rating <> QP_UNCHANGED then
          TQPROM(Selected[I]).Rating := _ROM.Rating;
        If _ROM.Language <> QP_UNCHANGED then
          TQPROM(Selected[I]).Language := _ROM.Language;
        If _ROM.Company <> QP_UNCHANGED then
          TQPROM(Selected[I]).Company := _ROM.Company;
        If _ROM.Multiplayer <> QP_UNCHANGED then
          TQPROM(Selected[I]).Company := _ROM.Multiplayer;
    End;
  End

  Else //else its only one selected and we can edit its values

  Begin

    try
      EditROM();
    except
      on E: EJException do
      begin
        MessageDlg(E.Message, mtError, [mbOK], 0);
        exit;
      end;
    end;    
  End;
  
  ROMlist.SaveToFile();

  end;//end of WITH block

  ModalResult := MrOk;
  
end;

procedure TFrmRomProp.BtnIPSdelClick(Sender: TObject);
begin
  if (IPSeditor.Cells[0, IPSeditor.Row] <> '') AND (IPSeditor.Cells[1, IPSeditor.Row] <> '')then
      IPSeditor.DeleteRow(IPSEditor.Row)
end;

procedure TFrmRomProp.BtnIPSinvalidClick(Sender: TObject);
Var
  I : Integer;
begin
  For I := IPSeditor.RowCount-1 downto 1 do
    If FileExists(IPSEditor.Cells[1,I]) = false then
      IPSEditor.DeleteRow(I);
end;

procedure TFrmRomProp.FormCreate(Sender: TObject);
begin
  Selected := TObjectList.Create(False);
  _ROM := TQPROM.Create;
  pgProp.ActivePageIndex := 0;
  VTData.RootNodeCount := 12;
end;

procedure TFrmRomProp.FormDestroy(Sender: TObject);
begin
  FreeAndNil(Selected);
  FreeAndNil(_ROM);
end;

procedure TFrmRomProp.FormShow(Sender: TObject);
begin
  Init;
end;

procedure TFrmRomProp.BtnIPSAddClick(Sender: TObject);
Var
  IPSFrm : TFrmIPS;
  IPSindex : Integer;
begin
  if not Multi then
  begin

      IPSFrm := TFrmIPS.Create(self);
      Try

      if IPSFrm.ShowModal = mrOK then
      begin
        //first resize the IPS array for the current ROM
        SetLength(_ROM.IPS, Length(_ROM.IPS)+1);
        IPSIndex := High(_ROM.IPS);
        _ROM.IPS[IPSIndex].index := IPSIndex;
        _ROM.IPS[IPSIndex].Name := IPSFrm.Name;
        _ROM.IPS[IPSIndex].Path := IPSFrm.Path;
        IPSeditor.InsertRow(IPSFrm.Name, IPSFrm.Path, true);
      end;

    Finally
      FreeAndNil(IPSFrm);
    end;

  end //end of 1 selected IF statement
end;


procedure TFrmRomProp.BtnNextClick(Sender: TObject);
begin
  try
    Move(True);
  except
    on e:EJException do MessageDlg(E.Message, mtError, [mbOK], 0);
  end;
end;

procedure TFrmRomProp.BtnPreviousClick(Sender: TObject);
begin
  try
    Move(False);
  except
    on e:EJException do MessageDlg(E.Message, mtError, [mbOK], 0);
  end;
end;

procedure TFrmRomProp.VTDataEditing(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; var Allowed: Boolean);
begin
  Allowed := (Column = 1);
end;

procedure TFrmRomProp.VTDataChange(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
begin
  with Sender do
  begin
    // Start immediate editing as soon as another node gets focused.
    if Assigned(Node) and not (tsIncrementalSearching in TreeStates) then
    begin
      // Note: the test whether a node can really be edited is done in the OnEditing event.
      EditNode(Node, 1);
    end;
  end;
end;

procedure TFrmRomProp.VTDataGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
begin
  if TextType = ttNormal then
    case Column of
      0:  Case Node.Index of
            0: CellText := QP_ROM_NAME;
            1: CellText := QP_ROM_PATH;
            2: CellText := QP_ROM_EMU;
            3: CellText := QP_ROM_COMPANY;
            4: CellText := QP_ROM_YEAR;
            5: CellText := QP_ROM_LANG;
            6: CellText := QP_ROM_RATING;
            7: CellText := QP_ROM_TYPE;
            8: CellText := QP_ROM_PLAYERS;
            9: CellText := QP_ROM_PARAM;
            10: CellText := QP_ROM_OVERRIDE_PARAM;
            11: CellText := QP_ROM_COMMENT;
          end;

      1:  Case Node.Index of
            0 : CellText := _ROM.Name;
            1 : CellText := _ROM.Path;
            2 : CellText := _ROM.Emulator;
            3 : CellText := _ROM.Company;
            4 : CellText := _ROM.Year;
            5 : CellText := _ROM.Language;
            6 : CellText := _ROM.Rating;
            7 : CellText := _ROM.GameType;
            8 : CellText := _ROM.Multiplayer;
            9 : CellText := _ROM.Parameters;
            10 : CellText := BoolToStr(_ROM.Overwrite, true);
            11 : CellText := _ROM.Comment;
          end;
    end;
end;

procedure TFrmRomProp.VTDataCreateEditor(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; out EditLink: IVTEditLink);
begin
  Case Node.Index of
    0: EditLink := TQPVTEdit.Create(qpeEdit);
    1:  begin
          //the path can either be a directory or a filename.
          if ExtractFileName(_ROM.Path) <> '' then
            EditLink := TQPVTEdit.Create(qpeFileEdit)
          else
            EditLink := TQPVTEdit.Create(qpeDirectoryEdit)
        end;
    2: EditLink := TQPVTEdit.Create(qpeComboEmu);
    3: EditLink := TQPVTEdit.Create(qpeEdit);
    4: EditLink := TQPVTEdit.Create(qpeEdit);
    5: EditLink := TQPVTEdit.Create(qpeComboLang);
    6: EditLink := TQPVTEdit.Create(qpeComboRating);
    7: EditLink := TQPVTEdit.Create(qpeComboType);
    8: EditLink := TQPVTEdit.Create(qpeEdit);
    9: EditLink := TQPVTEdit.Create(qpeEdit);
    10:EditLink := TQPVTEdit.Create(qpeComboBoolean);
    11:EditLink := TQPVTEdit.Create(qpeEdit);
  end;
end;

procedure TFrmRomProp.VTDataNewText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; NewText: WideString);
begin
  if Node <> nil then

    Case Node.Index of
      0:  if _ROM.Name <> NewText then
          begin
            _ROM.Name := NewText;
            _Changed := True;
          end;

      1:  if _ROM.Path <> NewText then
          begin
            _ROM.Path := NewText;
            _Changed := True;
          end;
          
      2:  if _ROM.Emulator <> NewText then
          begin
            _ROM.Emulator := NewText;
            _Changed := True;
          end;

      3:  if _ROM.Company <> NewText then
          begin
            _ROM.Company := NewText;
            _Changed := True;
          end;

      4:  if _ROM.Year <> NewText then
          begin
            _ROM.Year := NewText;
            _Changed := True;
          end;

      5:  if _ROM.Language <> NewText then
          begin
            _ROM.Language := NewText;
            _Changed := True;
          end;

      6:  if _ROM.Rating <> NewText then
          begin
            _ROM.Rating := NewText;
            _Changed := True;
          end;

      7:  if _ROM.GameType <> NewText then
          begin
            _ROM.GameType := NewText;
            _Changed := True;
          end;

      8:  if _ROM.Multiplayer <> NewText then
          begin
            _ROM.Multiplayer := NewText;
            _Changed := True;
          end;
          
      9:  if _ROM.Parameters <> NewText then
          begin
            _ROM.Parameters := NewText;
            _Changed := True;
          end;

      10: if _ROM.Overwrite <> StrToBool(NewText) then
          begin
            _ROM.Overwrite := StrToBool(NewText);   
            _Changed := True;
          end;

      11: if _ROM.Comment <> NewText then
          begin
            _ROM.Comment := NewText;
            _Changed := True;
          end;
    end;
end;

end.
