unit uEmuList;

interface

uses SysUtils, Controls, Contnrs, Classes, Dialogs, uRom, uEmu;

type

  TQPEmuToStrings = (cfAllEmus, cfMame, cfEmuBySys);

  TQPEmuList = class(TObjectList)
  protected
    _FileName : String;
    _AllowWrite : Boolean;
    function GetItem(Index: Integer): TQPEmu;
    procedure SetItem(Index: Integer; AObject: TQPEmu);
  public
    constructor Create(eFile : TFileName; AllowWrite : Boolean);
    Procedure DeleteAll(CfgDir : String);
    procedure DeleteEmulator(Index : Integer; AutoDel : Boolean; Favs : TObjectList;
          SettingsDir : String);
    Function  EmuExists(NewName : string): boolean;
    procedure EmusToStrings(const sList: TStrings; opt : TQPEmuToStrings; sys : string = '');
    function GetEmuForROM(ROM: Tobject): TQPEmu;
    function  IndexOfName(name : string): integer;
    function  IndexOfPath(path : string): integer;
    function  last(): TQPEmu;
    procedure LoadFromFile();
    procedure SaveToFile();
    function TotalForSystem(System : String):Integer;
    property Items[Index: Integer]: TQPEmu read GetItem write SetItem; default;
  end;

implementation

uses uQPConst, IniFiles, JCLStrings, uRomUtils, uRomList;

{$I Compilers.inc}

{ TQPEmuList }

constructor TQPEmuList.Create(eFile : TFileName; AllowWrite : Boolean);
begin
  OwnsObjects := True;
  _FileName := eFile;
  _AllowWrite := True;
end;

function TQPEmuList.GetItem(Index: Integer): TQPEmu;
begin
  Result := TQPEmu(inherited Items[Index]);
end;

{-----------------------------------------------------------------------------}

function TQPEmuList.last: TQPEmu;
begin
  Result := TQPEmu(inherited Items[Count-1]);
end;

{-----------------------------------------------------------------------------}

procedure TQPEmuList.SetItem(Index: Integer; AObject: TQPEmu);
begin
  inherited Items[Index] := AObject;
end;

{-----------------------------------------------------------------------------}

Procedure TQPEmuList.DeleteAll(CfgDir : String);
var
  I : Integer;
begin

   //Delete the emulator data file. 
   if fileexists(_FileName) then
      deletefile(_FileName);

   // There might be left over dat files from MAME renaming, remove them.
   for i := 0 to Self.Count-1 do
   begin
    if StrCompare(Self[i].system, 'arcade')=0 then
      if fileExists(CfgDir + Self[i].name + '.dat') then
        DeleteFile(CfgDir + Self[i].name + '.dat');
   end;

   //finally clear my list!
   Self.Clear;
end;

{-----------------------------------------------------------------------------}

procedure TQPEmuList.DeleteEmulator(Index : Integer; AutoDel : Boolean; Favs : TObjectList;
          SettingsDir : String);
Var
  CurEmu : TQPEmu;
Begin

  If Index > self.Count-1 then
    exit
  else
    CurEmu := self[Index];

  // firstly run through the linked roms delete function
  if AutoDel = true then
  Begin
    uRomUtils.DeleteAllByEmulator(CurEmu.name);
    TQPRomList(Favs).DeleteByEmu(CurEmu.name);
  end
  else
    if AutoDel = false then
    begin
      if MessageDlg(QP_DELETE_ALL_LINKED_ROMS,
             mtWarning, [mbYes, mbNo], 0) = mrYes then
      Begin
        uRomUtils.DeleteAllByEmulator(CurEmu.name);
        TQPRomList(Favs).DeleteByEmu(CurEmu.name);
      end;
    end;

  //Check if the emulator is a 'mame' to delete any dat files
  if StrCompare(CurEmu.system, 'arcade')=0 then
    if fileExists(SettingsDir+ CurEmu.name + '.dat') then
      DeleteFile(SettingsDir+ CurEmu.name + '.dat');

  //now do the actual deletion
  self.Delete(Index);

  //save the current list to disk.
  self.SaveToFile();

end;

{-----------------------------------------------------------------------------}

//checks to see if the emulator already exists in the array.
Function TQPEmuList.EmuExists(NewName : string): boolean;
var
  i : integer;
Begin
  result := false;

  For i := 0 to Self.Count-1 do
    if StrCompare(NewName, Self.Items[i].name)=0 then
    begin
      result := true;
      break;
    end;

end;

{-----------------------------------------------------------------------------}

procedure TQPEmuList.EmusToStrings(const sList: TStrings; opt : TQPEmuToStrings; sys : string = '');
var
  i : integer;
begin

  { Fills a combo box which is passed as a parameter with all the systems
    That are available from the system.dat file.
    cfAllEmus, cfMame, cfEmuBySys  }
    
  sList.Clear;

  Case opt of
    cfAllEmus:  begin
                  //Add all emulator names to combo box
                  for I:= 0 to self.Count-1 do
                    if Self[i].CmdLine or Self[i].ShellEx then
                      sList.add(self[i].name);
                end;

    cfMame:     begin
                //Add all Mame-mode emulators to a combo box.
                for I:= 0 to self.Count-1 do
                    If (self[i].mame) and (Self[i].CmdLine or Self[i].ShellEx) then
                      sList.add(self[i].name);
                end;

    cfEmuBySys: Begin
                //Add emulators which belong to a specific system
                for I := 0 to self.Count-1 do
                    If (self[i].system = sys) and (Self[i].CmdLine or Self[i].ShellEx) then
                      sList.Add(self[i].name);
                End;
  end;  //end of CASE statement.
end;

{-----------------------------------------------------------------------------}

function TQPEmuList.GetEmuForROM(ROM: Tobject): TQPEmu;
var
  i : Integer;
begin
  result := nil;

  for i := 0 to self.Count-1 do
    if StrCompare(self.Items[i].name, TQPROM(ROM).Emulator)=0 then
    begin
      result := self.Items[i];
      break;
    end;
end;

{-----------------------------------------------------------------------------}

//Function to figure out the position in the emulator array from the
//name of the emulator that is passed to it.  used for the Tree view
function TQPEmuList.IndexOfName(name : string): integer;
var
  i : integer;
begin

  result := -1;

  for i := 0 to Self.Count-1 do
    if StrCompare(Self[i].name, name)=0 then
    begin
      result := i;
      break;
    end;

end;

{-----------------------------------------------------------------------------}

function TQPEmuList.IndexOfPath(path : string): integer;
var
  i : integer;
begin

  result := -1;

  for i := 0 to Self.Count-1 do
    if StrCompare(Self[i].path, path)=0 then
    begin
      result := i;
      break;
    end;

end;

{-----------------------------------------------------------------------------}


Procedure TQPEmuList.LoadFromFile();
var
  Ini : TMemIniFile;
  i : integer;
  Sections : TStringList;
  NewEmu : TQPEmu;
begin

  self.Clear;

  //if there is no emulators file, set the array to 0 and exit
  if not FileExists(_FileName) then
  begin
    Exit;
  end;

  //create the INI object.
  Ini := TMemIniFile.Create(_FileName);

  //start the try-finally block
  try

    //create a stringlist to hold the total number of sections
    Sections := TStringList.Create;
    Try
      Ini.ReadSections(Sections);

      //take out the Internal data section.

      If Ini.SectionExists('QP Internal Data') then
        Sections.Delete(Sections.IndexOf('QP Internal Data'));

      self.Capacity := Sections.Count;

      For i := 0 to Sections.Count-1 do
      Begin
        //Now begin to set this records values from the ini file.
        self.Add(TQPEmu.Create);

        NewEmu := self.Last;

        NewEmu.name := Sections.Strings[i];
        NewEmu.Version := Ini.ReadString(Sections.Strings[i], 'Version', '');
        NewEmu.path := Ini.ReadString(Sections.Strings[i], 'Path','');
        NewEmu.cfg := Ini.ReadString(Sections.Strings[i], 'cfg','');
        NewEmu.system := Ini.ReadString(Sections.Strings[i], 'system','');
        NewEmu.WorkingDir := Ini.ReadString(Sections.Strings[i], 'WorkingDir','');
        NewEmu.HomePage := Ini.ReadString(Sections.Strings[i], 'HomePage','');
        NewEmu.parameters := Ini.ReadString(Sections.Strings[i], 'parameters','');
        NewEmu.ParamBefore := Ini.ReadBool(Sections.Strings[i], 'ParamBefore', false);
        NewEmu.CmdLine := Ini.ReadBool(Sections.Strings[i], 'CmdLine', true);
        NewEmu.UseLaunchApp := Ini.ReadBool(Sections[i], 'UseLaunchApp', False);
        NewEmu.ShellEx := Ini.ReadBool(Sections.Strings[i], 'ShellEx', false);
        NewEmu.zip := Ini.ReadBool(Sections.Strings[i], 'zip', false);
        NewEmu.mame := Ini.ReadBool(Sections.Strings[i], 'mame' , false);
        NewEmu.MameIncPath := Ini.ReadBool(Sections.Strings[i], 'MameIncPath', false);
        NewEmu.OnlyROMName := Ini.ReadBool(Sections.Strings[i], 'OnlyROMName', false);
        NewEmu.IncROMExt := Ini.ReadBool(Sections.Strings[i], 'IncROMExt', false);
        NewEmu.ShortExe := Ini.ReadBool(Sections.Strings[i], 'ShortExe', false);
        NewEmu.ShortRom := Ini.ReadBool(Sections.Strings[i], 'ShortRom', false);
        NewEmu.UseSpeechMarks := Ini.ReadBool(Sections.Strings[i], 'UseSpeechMarks', false);
        NewEmu.DisScrSvr := Ini.ReadBool(Sections.Strings[i], 'DisScrSvr', false);
        NewEmu.DisWinKey := Ini.ReadBool(Sections.Strings[i], 'DisWinKey', false);
        NewEmu.Verify := Ini.ReadBool(Sections.Strings[i], 'Verify', false);
        NewEmu.IconIndex := -1;
      End;

    //now free the stringlist containing the sections.
    Finally
      FreeAndNil(Sections);
    end;

  //end the try-finally block and free the ini object.
  finally
    FreeAndNil(ini);
  end;

end;

Procedure TQPEmuList.SaveToFile();
var
  Ini : TMemIniFile;
  i : integer;
  CurrEmu : TQPEmu;
begin

  //check that we are 'allowed' to write to the disk
  If not _AllowWrite then
    exit;

  //now create the ini object we will be using, and open a try-finally
  ini := TMemIniFile.Create(_FileName);
  try

    //before i write anythign to the ini file, erase existing information
    Ini.Clear;

    //first thing i want to do is add versioning information to the file.
    Ini.WriteString('QP Internal Data', 'Version', '3.6.2');
    Ini.WriteInteger('QP Internal Data', 'Total Records', self.Count);

    //the next section of code will loop through the emulators array
    For i := 0 to self.Count-1 do
    Begin
      CurrEmu := self[i];

      //check that the emulator has a valid name AND a valid Path
      if (CurrEmu.Name = '') or (CurrEmu.Path = '') then
        continue;

      Ini.WriteString(CurrEmu.Name, 'Version', CurrEmu.Version);
      Ini.WriteString(CurrEmu.Name, 'path', CurrEmu.path);
      Ini.WriteString(CurrEmu.Name, 'cfg', CurrEmu.cfg);
      Ini.WriteString(CurrEmu.Name, 'system', CurrEmu.system);
      Ini.WriteString(CurrEmu.Name, 'WorkingDir', CurrEmu.WorkingDir);
      Ini.WriteString(CurrEmu.Name, 'HomePage', CurrEmu.HomePage);
      Ini.WriteString(CurrEmu.Name, 'parameters', CurrEmu.parameters);
      Ini.writeBool(CurrEmu.Name, 'ParamBefore', CurrEmu.ParamBefore);
      Ini.WriteBool(CurrEmu.name, 'CmdLine', CurrEmu.CmdLine);
      Ini.WriteBool(CurrEmu.Name, 'UseLaunchApp', CurrEmu.UseLaunchApp);
      Ini.writeBool(CurrEmu.Name, 'ShellEx', CurrEmu.ShellEx);
      Ini.writeBool(CurrEmu.Name, 'zip', CurrEmu.zip);
      Ini.writeBool(CurrEmu.Name, 'mame', CurrEmu.mame);
      Ini.writeBool(CurrEmu.Name, 'MameIncPath', CurrEmu.MameIncPath);
      Ini.writeBool(CurrEmu.Name, 'IncROMExt', CurrEmu.IncROMExt);
      Ini.writeBool(CurrEmu.Name, 'OnlyROMName', CurrEmu.OnlyROMName);
      Ini.writeBool(CurrEmu.Name, 'ShortExe', CurrEmu.ShortExe);
      Ini.writeBool(CurrEmu.Name, 'ShortRom', CurrEmu.ShortRom);
      Ini.writeBool(CurrEmu.Name, 'UseSpeechMarks', CurrEmu.UseSpeechMarks);
      Ini.writeBool(CurrEmu.Name, 'DisScrSvr', CurrEmu.DisScrSvr);
      Ini.writeBool(CurrEmu.Name, 'DisWinKey', CurrEmu.DisWinKey);
      Ini.writeBool(CurrEmu.Name, 'Verify', CurrEmu.Verify);

    End;

    //finally since we are using a TMemIniFile, remember to call updatefile.
    if _AllowWrite then
      Ini.UpdateFile;

  // end of ini objects life span, free the object and exit the function.
  finally
    FreeAndNil(ini);
  end;

end;

{-----------------------------------------------------------------------------}

function TQPEmuList.TotalForSystem(System : String):Integer;
Var
  I, Count : Integer;
begin
  Count := 0;

  For i := 0 to self.Count-1 do
    if StrCompare(self[i].system, System)=0 then
      Inc(Count);

  Result := Count;
end;

{-----------------------------------------------------------------------------}

end.
 
