unit uJUtilities;

interface

uses
  Windows, Forms, SysUtils, Dialogs, Classes;

type
  EJException = class(Exception);
  EJWarning = class(Exception);
  EJNotice = class(Exception);

  {Function Declarations}
  Procedure AddToLog(Output : String);
  procedure OpenHomePage(HomePage: String);
  Function RunProcess(const AppPath:String; MustWait: Boolean=FALSE; CurrDir:String='';
    Visibility: Word=SW_SHOWNORMAL) : String;
  Function ScreensaverOnOff(Value : Boolean) : Boolean;
  Procedure WinKeyOnOff(Value : Boolean);

implementation

uses ShellAPI, JclStrings, uQPConst;

{$I Compilers.inc}

{-----------------------------------------------------------------------------}

Procedure AddToLog(Output : String);
Var
  Filename : string;
  LogFile : TextFile;
begin
  //prepare the log file
  FileName := ChangeFileExt(Application.ExeName, '.log');
  AssignFile(LogFile, filename);
  try
    If FileExists(FileName) then
      Append(LogFile)  //Open the existing log
    else
      Rewrite(LogFile);  //Create a new log

    //Write the file and show the error
    Writeln(LogFile, DateTimeToStr(now()) + ' : ' + Output);
  Finally
    //Close the File
    CloseFile(logFile);
  end;
end;

{-----------------------------------------------------------------------------}

procedure OpenHomePage(HomePage: String);
begin

  //this procedure opens the home page specified by the
  //string sent to the procedure

  //for security reasons we need to make sure that this is actually a URL!
  if (HomePage <> '') and
   ( (JCLStrings.StrIPos('http://', HomePage) <> 0) or
     (JCLStrings.StrIPos('www', HomePage) <> 0) or
     (JCLStrings.StrIPos('.co', HomePage) <> 0) ) then
    ShellExecute(Application.Handle, 'open', PChar(HomePage), '', '', sw_Show);

end;

{-----------------------------------------------------------------------------}

//Procedure to run a process - Dont know why i need this but i do.
function RunProcess(const AppPath:String; MustWait: Boolean=FALSE; CurrDir:String='';
  Visibility: Word=SW_SHOWNORMAL) : String;
var
  SI: TStartupInfo;
  PI: TProcessInformation;
  Proc: THandle;
  zFileName: array[0..255] of Char;
  zCurrPath: array[0..255] of Char;
  PResult : Dword;
begin

  Result := '';
  
  if AppPath = '' then
    Result := QP_RUN_PROCESS_EMPTY
  else
  Begin
  
  FillChar(SI, SizeOf(SI), 0);
  SI.cb := SizeOf(SI);
  SI.dwFlags:= StartF_UseShowWindow;
  SI.wShowWindow := Visibility;
  StrPCopy(zFileName, AppPath);
  //CREATE_NEW_PROCESS_GROUP   HIGH_PRIORITY_CLASS  Normal_Priority_Class
  if CurrDir <> '' then
  Begin
    StrPCopy(zCurrPath, CurrDir);
    if not CreateProcess(nil, zFileName,
        nil, nil, false, CREATE_NEW_PROCESS_GROUP+Normal_Priority_Class, nil, zCurrPath, SI, PI) then
      Result := QP_RUN_PROCESS_FAIL;
  end
  else
    if not CreateProcess(nil, zFileName,
        nil, nil, false, CREATE_NEW_PROCESS_GROUP+Normal_Priority_Class, nil, nil, SI, PI) then
      Result := QP_RUN_PROCESS_FAIL;

  Proc := PI.hProcess;
  CloseHandle(PI.hThread);

  if MustWait then
  begin
    // it will wait 10 seconds (10000)
    if WaitForSingleObject(Proc, 10000) <> Wait_Failed then
    begin
      GetExitCodeProcess(Proc, PResult);
      while PResult = STILL_ACTIVE do
        begin
          Application.ProcessMessages;
          if WaitForSingleObject(Proc, 10000) <> Wait_Failed then
            GetExitCodeProcess(Proc, PResult);
        end;
    end;
  end;

  CloseHandle(Proc);

  end;
end;

{-----------------------------------------------------------------------------}

Function ScreensaverOnOff(Value : Boolean) : Boolean;
Begin
  If Value then
    Result := SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 1, nil, 0)
  else
    Result := SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, 0, nil, 0);
End;

{-----------------------------------------------------------------------------}

Procedure WinKeyOnOff(Value : Boolean);
Var
  Dummy : Integer;
Begin
  If Value then
  //enable the win keys
  Begin
    SystemParametersInfo( SPI_SETFASTTASKSWITCH, 0, @Dummy, 0);
    SystemParametersInfo( SPI_SCREENSAVERRUNNING, 0, @Dummy, 0);
  End

  Else
  //disable the win keys
  Begin
    SystemParametersInfo( SPI_SETFASTTASKSWITCH, 1, @Dummy, 0);
    SystemParametersInfo( SPI_SCREENSAVERRUNNING, 1, @Dummy, 0);
  End;
End;


end.
