unit uJZip;

interface

uses SysUtils, Classes, ZipMstr;

    //these 2 functions are called when we need to create and destroy the zipmaster.
    procedure CreateZip();
    Procedure DestroyZip();

    Function GetZipVersion() : String;
    Procedure GetZipContents(var Contents : TStrings; ZipFile : TFileName);
    Procedure GetLastZipContents(var Contents : TStrings);
    procedure RenameContentsToZip(FileName : TFileName);
    procedure RenameZipToContents(FileName : TFileName);
    procedure UnzipSingleFile(ZipFile : TFileName; OutDir : String; ExtrFile : TFileName);
    Function UnzipROM(var ZipContents : TStringList; Zip : TFileName) : String;
    Procedure UnzipZip(ZipFile : TFileName; OutDir : String; Opt_Diff, Opt_Path, Opt_OW : boolean);
    Function VerifyZip(Z_File : TFileName) : Boolean;
    procedure ZipDirectory(Directory : String; ZipName : TFileName; Filter : TStrings);
    procedure ZipSingleFile(NewFile : TFileName; OutDir : String; CompLevel : Integer = 9);

implementation

{$I Compilers.inc}

uses JCLStrings, JCLSysInfo, uJUtilities, fMain;

var
  MyZipMstr : TZipMaster;

{-----------------------------------------------------------------------------}

procedure CreateZip();
begin
  MyZipMstr := TZipMaster.Create(nil);
  MyZipMstr.Load_Unz_Dll;
  MyZipMstr.Load_Zip_Dll;
end;

{-----------------------------------------------------------------------------}

Procedure DestroyZip();
begin
  MyZipMstr.Unload_Zip_Dll;
  MyZipMstr.Unload_Unz_Dll;
  FreeAndNil(MyZipMstr);
end;

{-----------------------------------------------------------------------------}

Procedure GetZipContents(var Contents : TStrings; ZipFile : TFileName);
var
  i : Integer;
begin

  MyZipMstr.ZipFileName := ZipFile;

  Contents.Clear;

  for i := 0 to MyZipMstr.Count-1 do
    Contents.Add(MyZipMstr.DirEntry[i].FileName);

end;

{-----------------------------------------------------------------------------}

Procedure GetLastZipContents(var Contents : TStrings);
var
  I : Integer;
begin
  Contents.Clear;
  For i := 0 to MyZipMstr.Count-1 do
    Contents.Add(MyZipMstr.dirEntry[i].FileName);
end;

{-----------------------------------------------------------------------------}

Function GetZipVersion() : String;
begin
  Result := MyZipMstr.VersionInfo;
end;

{-----------------------------------------------------------------------------}

procedure RenameContentsToZip(FileName : TFileName);
Var
  RenRec : pZipRenameRec;
  ZipRenameList : TList;
  TmpZip : ^ZipDirEntry;
  Path, OldName, NewName : String;
  RError, i : Integer;
Begin

  MyZipMstr.ZipFileName := FileName;

  if MyZipMstr.zipcontents.count = 1 then
  begin

    TmpZip := MyZipMstr.ZipContents.Items[0];

    OldName := TmpZip^.FileName;

    Path := ExtractFileExt(OldName);

    NewName := ChangeFileExt(ExtractFileName(FileName), Path);

    If StrCompare(ChangeFileExt(OldName,''), ChangeFileExt(NewName,'')) <> 0  then
    Begin
      New(RenRec);
      Try
        ZipRenameList := TList.Create;

        RenRec^.Source := OldName;
        RenRec^.Dest := NewName;
        RenRec^.DateTime := 0;

        ZipRenameList.Add(RenRec);

        RError := MyZipMstr.Rename(ZipRenameList, 0);

        if RError <> 0  then
          raise EJException.Create('Error ' + IntToStr( RError ) + ' occurred in rename zip specification(s)');

      Finally
        for i := 0 to ZipRenameList.Count - 1 do
        begin
          RenRec := ZipRenameList.Items[i];
          Dispose( RenRec );
        end;

        FreeAndNil(ZipRenameList);
      end;

    End;

  end;
End;

{-----------------------------------------------------------------------------}

procedure RenameZipToContents(FileName : TFileName);
Var
  Name : String;
  TmpZip : ^ZipDirEntry;
Begin
  MyZipMstr.ZipFileName := FileName;

  if MyZipMstr.zipcontents.count = 1 then
  begin
    TmpZip := MyZipMstr.ZipContents.Items[0];

    name := TmpZip^.FileName;
    name := ChangeFileExt(name,'.zip');
    if not renameFile(FileName, name) then
      raise EJException.Create('Error renaming file to' + Name);
  end
  else
    Raise EJWarning.Create('Zip contained more than 1 file');

End;

{-----------------------------------------------------------------------------}

Function UnzipROM(var ZipContents : TStringList; Zip : TFileName) : String;
var
  i : Integer;
  UseDir, OldCurrent : String;
begin

  Result := '';

  If MainFrm.Settings.AllowWrite then
    UseDir := MainFrm.Settings.QPDirectory + 'temp\'
  else
    UseDir := ExtractFilePath(Zip);

  //now we will test if we can actually extract this file
  OldCurrent := GetCurrentDir();
  SetCurrentDir(UseDir);
  try
    If DiskFree(0) = 0 then
      UseDir := IncludeTrailingPathDelimiter(GetWindowsTempFolder()) + 'qp\temp\';
  finally
    SetCurrentDir(OldCurrent);
  end;

  {$IFDEF DEBUG}
  AddToLog('ROM will be extracted to : ' + UseDir);
  {$ENDIF}

  //now create the temp directory again.
  If not DirectoryExists(UseDir) then
    ForceDirectories(UseDir);

  try
    UnzipZip(Zip, UseDir, False, False, true);
    
    For i := 0 to MyZipMstr.Count-1 do
      ZipContents.Add(UseDir + ExtractFileName(MyZipMstr.DirEntry[i].FileName));

    //if one file was extracted this is the rom, save the path
    If MyZipMstr.Count = 1 then
      Result := UseDir + ExtractFileName(MyZipMstr.DirEntry[0].FileName);

    //if more than one file is extracted we have a problem!
    //Just try to GUESS which file is the rom... pants...
    If MyZipMstr.Count > 1 then
      For i := 0 to MyZipMstr.Count-1 do
        If (ExtractFileExt(MyZipMstr.DirEntry[i].FileName) <> '.txt') or
           (ExtractFileExt(MyZipMstr.DirEntry[i].FileName) <> '.nfo') or
           (ExtractFileExt(MyZipMstr.DirEntry[i].FileName) <> '.diz') then
        Begin
          Result := UseDir + ExtractFileName(MyZipMstr.DirEntry[i].FileName);
          Break;
        End;
  except
    Result := '';
  end;
end;

{-----------------------------------------------------------------------------}

{ This Function will take a zip file and extract it.  There are a lot of
  arguments needed in this function.  These are : firstly a zipmaster component
  Then the zip file, and the directory you wish to output the file.  Next are 3
  boolean values, to determine if you want a seperate directory for this file,
  whether you want to extract with path, and if you wish to overwrite.}
procedure UnzipZip(ZipFile : TFileName; OutDir : String; Opt_Diff, Opt_Path, Opt_OW : boolean);
var
  temp : string;
begin
  MyZipMstr.ZipFileName := ZipFile;

  with MyZipMstr do
  begin

    if Opt_Diff = true then
    begin
      temp := ExtractFileName(ZipFile);
      Temp := ChangeFileExt(Temp, '');
      temp := IncludeTrailingPathDelimiter(OutDir) + Temp;

      if (Not DirectoryExists(Temp)) then
        CreateDir(temp);

      ExtrBaseDir := IncludeTrailingPathDelimiter(temp);
    end
    else
      ExtrBaseDir := IncludeTrailingPathDelimiter(OutDir);
                                             
    SetCurrentDir(ExtrBaseDir);

    If Opt_OW Then
      ExtrOptions:=ExtrOptions+[ExtrOverwrite]
    else
      ExtrOptions:=ExtrOptions-[ExtrOverwrite];

    If Opt_Path Then
      ExtrOptions:=ExtrOptions+[ExtrDirNames]
    else
      ExtrOptions:=ExtrOptions-[ExtrDirNames];

    FSpecArgs.Add('*.*');

    Try
      Extract;
    except
      Raise EJException.Create('Error in Extract; DLL Error : ' + IntToStr(MyZipMstr.ErrCode));
    end;
  end;
end;

{-----------------------------------------------------------------------------}

{ This function takes a zip file and extracts one single file from it.  }
procedure UnzipSingleFile(ZipFile : TFileName; OutDir : String; ExtrFile : TFileName);
begin
  MyZipMstr.ZipFileName := ZipFile;
  MyZipMstr.ExtrBaseDir := IncludeTrailingPathDelimiter(OutDir);

  SetCurrentDir(MyZipMstr.ExtrBaseDir);

  MyZipMstr.ExtrOptions := MyZipMstr.ExtrOptions+[ExtrOverwrite];
  MyZipMstr.ExtrOptions := MyZipMstr.ExtrOptions-[ExtrDirNames];

  MyZipMstr.FSpecArgs.Clear;
  MyZipMstr.FSpecArgs.Add(ExtrFile);
  MyZipMstr.Extract;
  MyZipMstr.FSpecArgs.Clear;
end;

{-----------------------------------------------------------------------------}

{This Function tests a given ZIP file to see if it is a valid archive.
 If the archive is valid, the function returns a true value, else it
 returns a false value if the zip file is invalid.  As well as passing
 the zip file to the function, you should pass a TZipMaster component.}
Function VerifyZip(Z_File : TFileName) : Boolean;
Begin

  Result := True;
  // Turn off the error messages
  MyZipMstr.Unattended := True;
  Try

    MyZipMstr.ZipFileName := Z_File;
    // If the zip file is invalid, then the last error message will
    // be a 11036 message.
    if MyZipMstr.ErrCode = 11036 then
      Result := False;

  Finally
    MyZipMstr.Unattended := False;
  end;

End;

{-----------------------------------------------------------------------------}

{This function will take a standard file and process its name to create a
 zip file with an identical name.  1 standard file per zip!}

procedure ZipSingleFile(NewFile : TFileName; OutDir : String; CompLevel : Integer = 9);
Var
  Temp : String;
begin
  temp := ExtractFileName(NewFile);
  Temp := ChangeFileExt(Temp, '');
  temp := OutDir + '\' + Temp + '.zip';
  MyZipMstr.ZipFileName := temp;
  MyZipMstr.FSpecArgs.Add(NewFile);
  MyZipMstr.AddCompLevel := CompLevel;

  try
    MyZipMstr.Add;
  except
    Raise EJException.Create('Error in Add; Fatal DLL Exception in mainunit');
  end;
end;

procedure ZipDirectory(Directory : String; ZipName : TFileName; Filter : TStrings);
var
  OldSettings : AddOpts;
begin
 //copy existing settings to a temp variable to reset later.
  OldSettings := MyZipMstr.AddOptions;
  try

    MyZipMstr.ZipFileName := ZipName;
    //set up the zip component to the way we need it.
    MyZipMstr.RootDir := Directory;
    MyZipMstr.AddOptions := MyZipMstr.AddOptions + [AddForceDest];
    MyZipMstr.AddOptions := MyZipMstr.AddOptions + [AddRecurseDirs];
    MyZipMstr.AddOptions := MyZipMstr.AddOptions + [AddDirNames];
    MyZipMstr.AddOptions := MyZipMstr.AddOptions + [AddSeparateDirs];

    MyZipMstr.FSpecArgs.Clear;
    MyZipMstr.FSpecArgs.AddStrings(Filter);
    MyZipMstr.AddCompLevel := 9;

    try
      MyZipMstr.Add;
    except
      Raise EJException.Create('Error compressing Data : ' + IntToStr(MyZipMstr.FullErrCode));
    end;

  finally
    //remember to turn off the options we set above.
    MyZipMstr.rootDir := '';
    MyZipMstr.AddOptions := OldSettings;
  end;
end;

{-----------------------------------------------------------------------------}

end.
