unit uRomList;

interface

uses Contnrs, Classes, SysUtils, ComCtrls, IniFiles, uRom, uEmuList;

Type

  TQPExportMode = (emAll, emSys, emEmu, emMissing, emAllDirs);
  TQPExportFormat = (efHTML, efXML, efText);
  TQPLanguageMode = (lmEnglish, lmNone, lmGuess);

  { List Of TQPRom Objects }
  TQPROMlist = class(TObjectList)
  protected
    _FileName : String;
    _AllowWrite : Boolean;
    function GetItem(Index: Integer): TQPRom;
    procedure SetItem(Index: Integer; AObject: TQPRom);
  public
    Constructor Create(AllowWrite : Boolean);
    Procedure AddFromDirs(lstDirs : TStrings; ff_Filter : String;
          ff_Recurse : Boolean; ScanZips : Boolean; EmuName : String;
          Rating : String; Language : TQPLanguageMode; NoDuplicates : Boolean);overload;
    procedure AddFromDirs(Dir : String; ff_Filter : String;
          ff_Recurse : Boolean; ScanZips : Boolean; EmuName : String;
          Rating : String; Language : TQPLanguageMode; NoDuplicates : Boolean);overload;
    Procedure AddFromStrings(RomStrings : TStrings; EmuName : String;
          Rating : String; Language : TQPLanguageMode; NoDuplicates : Boolean);
    procedure AddFromZip(ZipFile : TFileName; EmuName : String; Rating : String;
          Language : TQPLanguageMode; NoDuplicates : Boolean);
    Function ChangeEmus(OldEmu, NewEmu : String): Integer;
    Function DeleteByEmu(EmuStr : String) : Integer;
    Function DeleteOrphans() : Integer;
    Procedure ExportCustomData(outFile : TFileName; IncCompany, IncType, IncLang, IncYear,
          IncRating, IncPlayers : Boolean);
    Function ExportRoms(CList : TStrings; Format : TQPExportFormat; Opt : TQPExportOpt;
          EList : TQPEmuList; Mode : TQPExportMode; Filter : String = '') : Integer;
    procedure ImportCustomData(inFile : TFileName; Overwrite, IncCompany, IncType, IncLang,
          IncYear, IncRating, IncPlayers : Boolean);
    function last(): TQPRom;
    procedure LoadFromFile(In_file : TFileName);
    procedure LoadMatchesFromFile(In_file : TFileName; Query : TObjectList;
          MatchAll : Boolean);
    Procedure MameCompareNames(Ini : TMemIniFile; Prog : TProgressBar=nil);
    procedure MirrorDirectories(Ini : TMemIniFile);
    procedure MoveTo(Source, Destination: Integer);
    Procedure Query(var MatchList : TObjectList; smArray : TObjectList; MatchAll : Boolean);
    Function ROMExists(Path : String; zipname : string = ''):boolean;
    procedure RunRandom(EList : TQPEmuList); 
    procedure SaveToFile();
    procedure SetMameCategories(SettingsDir : String);
    procedure SetMameMultiPlayer(SettingsDir : String);

    property Items[Index: Integer]: TQPRom read GetItem write SetItem; default;
    Property FileName : String Read _FileName write _FileName;
    Property AllowWrite : Boolean Read _AllowWrite write _AllowWrite;
  end;

implementation

{$I Compilers.inc}

uses JclFileUtils, JCLStrings, FindFile, uEmu,
  uJUtilities, uJZip, uQuery, uQPConst;

{ TQPROMlist }

Constructor TQPROMlist.Create(AllowWrite : Boolean);
begin
  OwnsObjects := True;
  _AllowWrite := AllowWrite;
end;

function TQPROMlist.GetItem(Index: Integer): TQPRom;
begin
  Result := TQPRom(inherited Items[Index]);
end;

{-----------------------------------------------------------------------------}

function TQPROMlist.last: TQPRom;
begin
  Result := TQPROM(inherited Items[Count-1]);
end;

{-----------------------------------------------------------------------------}

procedure TQPROMlist.SetItem(Index: Integer; AObject: TQPRom);
begin
  inherited Items[Index] := AObject;
end;

{-----------------------------------------------------------------------------}

Procedure TQPROMList.AddFromDirs(Dir : String; ff_Filter : String;
          ff_Recurse : Boolean; ScanZips : Boolean; EmuName : String;
          Rating : String; Language : TQPLanguageMode; NoDuplicates : Boolean);
var
  LstDirs : TStringList;  
begin
  //This is just a wrapper for the other version, which takes a string for a directory
  //name.  The string is added to a stringlist and then the other version is called.
  LstDirs := TStringList.Create;
  try
    LstDirs.Add(Dir);
    AddFromDirs(LstDirs, ff_Filter, ff_Recurse, ScanZips, EmuName, Rating, Language,
                NoDuplicates);
  finally
    FreeAndNil(LstDirs);
  end;
end;

Procedure TQPRomList.AddFromDirs(lstDirs : TStrings; ff_Filter : String;
          ff_Recurse : Boolean; ScanZips : Boolean; EmuName : String;
          Rating : String; Language : TQPLanguageMode; NoDuplicates : Boolean);
var
  Find : ThhFindFile;
  LstFound : TStringList;
  i : Integer;
begin
  if (lstDirs.Count > 0) then
  begin

      LstFound := TStringList.Create;
      try

        //set the sorted property to true.
        LstFound.Sorted := True;

        Find := ThhFindFile.Create(nil);
        try
        
          //set up the search options
          Find.filter := ff_filter;
          Find.recurse := ff_Recurse;

          for i := 0 to lstDirs.Count-1 do
          begin
            Find.Directory := lstDirs[i];
            Find.execute;
            //add the results to our stringlist.
            LstFound.AddStrings(Find.Files);
          end;

        Finally
          FreeAndNil(Find);
        end;

        //now we have a list of search results, we can save them

        If ScanZips then
          For i := LstFound.Count-1 downto 0 do
          begin
            Self.AddFromZip(LstFound[i], EmuName, Rating, Language, NoDuplicates);
            LstFound.Delete(i);
          end;

        //now save whatever is left over
        Self.AddFromStrings(LstFound, EmuName, Rating, Language, NoDuplicates);
      Finally
        FreeAndNil(LstFound);
      end;
  end;

end;

Procedure TQPROMlist.AddFromStrings(RomStrings : TStrings; EmuName : String;
          Rating : String; Language : TQPLanguageMode; NoDuplicates : Boolean);
var
  i :integer;
  NewRom : TQPRom;
Begin
  // Add all Roms in the stringlist to the current list.
  Self.Capacity := Self.Count + RomStrings.Count;
  
  for i := 0 to RomStrings.Count-1 do
  begin

    If NoDuplicates then
        if Self.ROMExists(RomStrings[i]) then
          Continue;

    Self.add(TQPRom.Create);
    NewROM := Self.Last();

    NewROM.name := ChangeFileExt(ExtractFileName(RomStrings[i]), '');
    NewROM.path := trim(RomStrings[i]);
    NewROM.emulator := EmuName;
    NewROM.Year := '';
    NewROM.NumPlay := 0;
    NewROM.gametype := '';
    NewROM.Parameters := '';
    NewROM.Overwrite := False;

    NewROM.Rating := Rating;

    case Language of
      lmEnglish : NewRom.Language := QP_ENG;
      lmNone : NewRom.Language := '';
      lmGuess : NewRom.SetLanguage();
    end;

  end;  //End of FOR loop

  // save the data
  Self.SaveToFile();

End;

{-----------------------------------------------------------------------------}

procedure TQPROMlist.AddFromZip(ZipFile : TFileName;
          EmuName : String; Rating : String; Language : TQPLanguageMode;
          NoDuplicates : Boolean);
var
  i :integer;
  NewRom : TQPRom;
  ROMname : String;
  ZipContents : TStrings;
Begin

  //first make sure that this file is indeed a zip file!
  If StrCompare('.zip', ExtractFileExt(ZipFile))=0 then
  begin
    ZipContents := TStringList.Create;
    try

    GetZipContents(ZipContents, ZipFile);

    for i := 0 to ZipContents.Count-1 do
    begin

      ROMname := ChangeFileExt(ExtractFileName(ZipContents[i]), '');

      If NoDuplicates then
        if Self.ROMExists(ZipFile, ZipContents[i]) = True then
          Continue;

      Self.add(TQPRom.Create);
      NewROM := Self.Last();

      NewROM.name := ROMname;
      NewROM.path := ZipFile;
      NewROM.ZipName := ZipContents[i];
      NewROM.emulator := EmuName;
      NewROM.Year := '';
      NewROM.NumPlay := 0;
      NewROM.gametype := '';
      NewROM.Parameters := '';
      NewROM.Overwrite := False;

      NewROM.Rating := Rating;

      case Language of
        lmEnglish : NewRom.Language := QP_ENG;
        lmNone : NewRom.Language := '';
        lmGuess : NewRom.SetLanguage();
      end;
      
    end;  //End of FOR loop

    Finally
      FreeAndNil(ZipContents);
    end;

  end;//end of IF is a zip file statement

  // save the data
  Self.SaveToFile();
End;


{-----------------------------------------------------------------------------}

Function TQPROMlist.ChangeEmus(OldEmu, NewEmu : String): Integer;
var
  i : Integer;
  Changed : boolean;
begin
  // Function to change associated emulator for entire list.

  Changed := False;
  Result := 0;

  for i := 0 to self.Count-1 do
    if StrCompare(self[i].emulator, OldEmu)=0 then
    begin
      self[i].emulator := NewEmu;
      changed := True;
    end;

  //If the list has been altered then write to disk.
  if changed then
    Self.SaveToFile();
end;

{-----------------------------------------------------------------------------}

Function TQPROMlist.DeleteByEmu(EmuStr : String) : Integer;
var
  I : Integer;
  Changed : Boolean;
begin
  // Function to Delete all roms in the list if they use a specified emulator.

  changed := False;
  Result := 0;
  for i := self.Count-1 downto 0 do
    //Find Roms linked to the deleted emulator and delete
    if StrCompare(self[i].emulator, EmuStr)=0 then
    begin
      self.Delete(i);
      changed := True;
    end;

  //If the roms have been altered then write the array to disk.
  if changed then
    Self.SaveToFile();
end;

{-----------------------------------------------------------------------------}

Function TQPROMlist.DeleteOrphans() : Integer;
var
   i : integer;
   Changed : Boolean;
Begin

  Changed := False;
  Result := 0;

  For i := Self.Count-1 downto 0 do
  if (not fileExists(Self[i].path)) and (not DirectoryExists(Self[i].Path)) then
  begin
    Self.Delete(i);
    inc(Result);
    changed := true;
  end;

  //If the roms have been altered then write the array to disk.
  if (changed) then
    Self.SaveToFile();

End;

{-----------------------------------------------------------------------------}

Procedure TQPROMlist.ExportCustomData(outFile : TFileName; IncCompany, IncType,
          IncLang, IncYear, IncRating, IncPlayers : Boolean);
var
  i : Integer;
  data : TStringList;
begin
  //This function outputs a ROMs information as an ini file for scanning in QP
  data := TStringList.Create;
  try

    for i := 0 to Self.Count-1 do
    begin

      Data.Add('[' + Self[i].FileNameNoExt + ']');
      if (IncCompany) and (Self[i].Company <> '') then
        Data.Add('Company=' + Self[i].Company);

      if (IncType) and (Self[i].GameType <> '') then
        Data.Add('Type=' + Self[i].GameType);

      if (IncLang) and (Self[i].Language <> '') then
        Data.Add('Language=' + Self[i].Language);

      if (IncYear) and (Self[i].Year <> '') then
        Data.Add('Year=' + Self[i].Year);

      if (IncRating) and (Self[i].Rating <> '') then
        Data.Add('Rating=' + Self[i].Rating);

      if (incPlayers) and (Self[i].Multiplayer <> '') then
        Data.Add('Players=' + Self[i].Multiplayer);

      Data.Add('');
    end;

    Data.SaveToFile(outFile);

  Finally
    FreeAndNil(Data);
  end;
end;

{-----------------------------------------------------------------------------}

Function TQPROMlist.ExportRoms(CList : TStrings; Format : TQPExportFormat; Opt : TQPExportOpt;
          EList : TQPEmuList; Mode : TQPExportMode; Filter : String = '') : Integer;
var
  i, e : Integer;
begin

  Result := 0;
  
  Case Mode of
    emAll, emAllDirs : //export all ROMs in the list.
      Begin
        Result := Self.Count;
        For i := 0 to Self.Count-1 do
        begin
          Case Format of
            efHTML : Self[i].ToHTML(CList, EList, Opt);
            efXML : Self[i].ToXML(CList, EList, Opt);
            efText : Self[i].ToCSV(CList, Opt);
          end;
        end;
      end;

    emSys : //export ROMs that are in the specific system.
      Begin
        if Filter <> '' then
          for i := 0 to Self.Count-1 do
            for e := 0 to EList.Count-1 do
              if StrCompare(Self[i].emulator, EList[e].name)=0 then
                if StrCompare(EList[e].system, Filter)=0 then
                begin
                  Case Format of
                    efHTML : Self[i].ToHTML(CList, EList, Opt);
                    efXML : Self[i].ToXML(CList, EList, Opt);
                    efText : Self[i].ToCSV(CList, Opt);
                  end;
                  Inc(Result);
                  break;
                end;
      end;

    emEmu : //Export ROMS associated to specific emulator.
      Begin
        if Filter <> '' then
          For i := 0 to Self.Count-1 do
            if StrCompare(Self[i].emulator, filter)=0 then
            begin
              Case Format of
                efHTML : Self[i].ToHTML(CList, EList, Opt);
                efXML : Self[i].ToXML(CList, EList, Opt);
                efText : Self[i].ToCSV(CList, Opt);
              end;
              Inc(Result);
            end;
      End;
  end;
  
end;

{-----------------------------------------------------------------------------}

procedure TQPROMlist.ImportCustomData(inFile : TFileName; Overwrite, IncCompany, IncType, IncLang,
          IncYear, IncRating, IncPlayers : Boolean);
var
  i : Integer;
  ini : TMemIniFile;
begin
  if FileExists(inFile) then
  begin

    ini := TMemIniFile.Create(inFile);
    try

      for i := 0 to Self.Count-1 do
      begin

        if (IncCompany) and ((Self[i].Company = '') or (overwrite)) then
          Self[i].Company := Ini.ReadString(Self[i].FileNameNoExt, 'Company', Self[i].Company);
        if (IncType) and ((Self[i].GameType = '') or (overwrite)) then
          Self[i].GameType := Ini.ReadString(Self[i].FileNameNoExt, 'Type', Self[i].GameType);
        if (IncLang) and ((Self[i].Language = '') or (overwrite)) then
          Self[i].Language := Ini.ReadString(Self[i].FileNameNoExt, 'Language', Self[i].Language);
        if (IncYear) and ((Self[i].Year = '') or (overwrite)) then
          Self[i].Year := Ini.ReadString(Self[i].FileNameNoExt, 'Year', Self[i].Year);
        if (IncRating) and ((Self[i].Rating = '') or (overwrite)) then
          Self[i].Rating := Ini.ReadString(Self[i].FileNameNoExt, 'Rating', Self[i].Rating);
        if (IncPlayers) and ((Self[i].Multiplayer = '') or (overwrite)) then
          Self[i].Multiplayer := Ini.ReadString(Self[i].FileNameNoExt, 'Players', Self[i].Rating);

      end;

      //save the updated list
      Self.SaveToFile;

    finally
      FreeAndNil(ini);
    end;

  end;
end;

{-----------------------------------------------------------------------------}

procedure TQPROMlist.LoadFromFile(In_file : TFileName);
var
  i : integer;
  filelist : TStringList;
begin

  //to start with we clear the ROM list.
  Self.Clear;
  Self.Pack;
  _FileName := In_file;

  if fileExists(In_file) AND (GetSizeOfFile(In_file) > 0) then
  begin

    filelist := TStringList.create;
    try
      filelist.LoadFromFile(In_file);

      //set the capacity of the ROM list to improve performance.
      Self.Capacity := FileList.Count;

      For i := 0 to FileList.Count-1 do
        // now add our new rom to the list.
        Self.Add(TQPRom.Create(FileList[i]));

    finally
      FreeAndNil(filelist);
    end;

  end;

  {$IFDEF DEBUG}
    filelist := TStringList.Create;
    try
      For i := 0 to Self.Count-1 do
        FileList.Add(Self[i].Name + ' - ' + Self[i].CloneName + ' - ' + Self[i].ParentName);

      FileList.SaveToFile(ParamStr(0) + 'DebugROMS.txt');
    Finally
      FreeAndNil(FileList);
    end;

  {$ENDIF}

end;

procedure TQPROMlist.LoadMatchesFromFile(In_file : TFileName; Query : TObjectList;
          MatchAll : Boolean);
var
  i, e : integer;
  filelist : TStringList;
  iQuery : TQPQuery;
  qResult : Boolean;
begin

  //to start with we clear the ROM list.
  Self.Clear;
  Self.Pack;
  _FileName := In_file;

  if fileExists(In_file) AND (GetSizeOfFile(In_file) > 0) then
  begin

    filelist := TStringList.create;
    try
      filelist.LoadFromFile(In_file);

      //set the capacity of the ROM list to improve performance.
      Self.Capacity := FileList.Count;

      For i := 0 to FileList.Count-1 do
      begin

        //now we do a quick String Comparison to see if any of the matches are in this line
        //before we turn it into a ROM.
        For e := 0 to Query.count-1 do
        begin
          iQuery := TQPQuery(Query[e]);
          qResult := JCLStrings.StrIPos(iQuery.Filter, FileList[i]) <> 0;
          //If we only need to match one element and its found then exit.
          if (qResult) and (not MatchAll) then
            Break;

          //we need to match all the criteria, and it hasnt (hence the false), so no
          //need to carry on in the loop.
          if (not qResult) and (MatchAll) then
            Break;

        end;

        // now add our new rom to the list.
        if qResult then
          Self.Add(TQPRom.Create(FileList[i]));
      end;

    finally
      FreeAndNil(filelist);
    end;

  end;
end;

{-----------------------------------------------------------------------------}
Procedure TQPROMlist.MameCompareNames(Ini : TMemIniFile; Prog : TProgressBar=nil);
var
  i : Integer;
  Temp : String;
begin

  //initialise the progress bar if there is one.
  if Prog <> nil then
  begin
    Prog.Position := 0;
    Prog.Max := Self.Count;
  end;

  //loop through every ROM looking for a rename
  for i := 0 to Self.Count-1 do
  begin

    //get the name of the ROM -without extension or path
    If Self[i].CloneName <> '' then
      Temp := Self[i].CloneName
    else
    begin
      Temp := Self[i].Name;
      Self[i].CloneName := Self[i].Name;
    end;
    //all MAME file names must be under 8 characters, this is a

    //simple to check to speed up the process a bit.
    if Length(Temp) <= 8 then
    begin
      Self[i].name := ini.ReadString(trim(temp), 'name' , Self[i].name);
      Self[i].Year := ini.ReadString(trim(temp), 'year' , Self[i].year);
    end;

    if Prog <> nil then
      //we have completed a ROM so step the progress bar
      Prog.StepIt;

  end; //end of FOR loop

  //now save this updated Emufile
  Self.SaveToFile();
end;

procedure TQPROMlist.MirrorDirectories(Ini : TMemIniFile);
var
  Find : ThhFindFile;
  I, FIndex : Integer;
  Directory, Emulator : string;
  FilterList, FindList, DirList : TStringlist;
  FilterStream : TMemoryStream;
begin

  Find := ThhFindFile.Create(nil);
  Try

    //check that mirroring is enabled, if its not then exit
    If Ini.ReadBool('Mirror', 'ChkMirror', false)=false then
      raise EJException.Create(QP_MIRROR_NOT_ENABLED);

    Directory := Ini.ReadString('Mirror', 'TxtDir', '');
    If Directory = '' then
      raise EJException.Create(QP_MIRROR_NO_DIRECTORIES);

    //now we need the filter string(!)
    FilterStream := TMemoryStream.Create;
    Try
      FilterList := TStringList.Create;
      Ini.ReadBinaryStream('Mirror', 'LstFilter', FilterStream);
      FilterList.LoadFromStream(FilterStream);
      Find.Filter := '';
      for i := 0 to (filterlist.Count - 1) do
        Find.Filter := Find.Filter + ';' + filterlist.Strings[i];

    Finally
      FreeAndNil(FilterStream);
      FreeAndNil(FilterList);
    end;

    //finally get the emulator
    Emulator := Ini.ReadString('Mirror', 'CmbEmu', '');

    //now finalise the Find setup and run it
    Find.Recurse := False;

    //create a stringlist to hold ALL found files from our scans
    FindList := TStringList.Create;
    try

      //create a stringlist to hold all the directories to scan
      DirList := TStringList.Create;
      try

        //parse the directory string into seperate - usuable directories.
        StrTokenToStrings(Directory,';', DirList);

        //now execute a search on each directory and
        //amalgamate all the results into one stringlist.
        For i := 0 to DirList.Count-1 do
        begin
          Find.Directory := DirList.Strings[i];
          Find.Execute;
          FindList.AddStrings(Find.Files);
        end;

      Finally
        FreeAndNil(DirList);
      end;

      //now we have all the files found in the directories in findlist
      //loop through all the roms and see if it occurs in the findlist
      //if it doesnt then remove the rom, if it does then remove
      //the entry in the findlist.
      For i := Self.Count-1 downto 0 do
      begin
        FIndex := FindList.IndexOf(Self[i].Path);
        if (FIndex = -1) then
          Self.Delete(i)
        else
          FindList.Delete(FIndex);
      end;

      //by now we have eliminated any entry in the ROMS array that
      //isnt in the findlist.  and any entry in FindList that was
      //already in the ROMlist has been removed also, so finally
      //just loop through and add the results in findlist to romlist.
      AddFromStrings(FindList, Emulator, '', lmNone, False);

    Finally
      FreeAndNil(FindList);
    end;

  Finally
    FreeAndNil(Find);
  end;

  Self.SaveToFile();
end;

{-----------------------------------------------------------------------------}

procedure TQPROMlist.MoveTo(Source, Destination : Integer);
begin

  //Wrapper for the move function, which checks users arent doing daft things.

  If (Source < 0) or (Source > self.Count) then
    exit;
  If (Destination < 0) or (Destination > self.Count) then
    exit;

  If Source <> Destination then
    self.Move(Source, Destination);

end;

{-----------------------------------------------------------------------------}

Procedure TQPROMlist.Query(var MatchList : TObjectList; smArray : TObjectList; MatchAll : Boolean);
var
  i : Integer;
begin

  for i := 0 to Self.Count-1 do
    if Self[i].MatchesQuery(smArray, MatchAll) then
      MatchList.Add(Self[i]);

end;

{-----------------------------------------------------------------------------}

// checks to see if this rom already exists in the currently loaded ROMSlist.
// This is done by filepath.  Also needs to take into account roms inside a
// zip file, as roms inside a zip can have the same path (to the zip)
// but will have a different internal ROM name.
function TQPROMlist.ROMExists(Path : String; zipname : string = ''):boolean;
var
  i : integer;
begin

  result := false;

  //If a zipname parameter was supplied, this means we need to check for
  //duplicates which are inside a zip file (thus the filepaths will be the same)
  If ZipName = '' then
  begin
    for i:= 0 to self.Count-1 do
      if (StrCompare(path, self[i].path) = 0) then
      Begin
        result := true;
        Break;
      End;
  end
  else
  begin
    for i := 0 to self.Count-1 do
      if StrCompare(ZipName, self[i].ZipName) = 0 then
      begin
        result := true;
        break;
      end;
  end;

end;

{-----------------------------------------------------------------------------}

procedure TQPROMlist.RunRandom(EList : TQPEmuList);
var
  RandROM : Integer;
  Emu : TQPEmu;
begin

  //Check that the ROMS array actually has entries.
  If Self.Count > 0 then
  begin

    //Initialise the random number generator.
    Randomize;

    RandROM := Random(Self.Count-1);

    Emu := EList.GetEmuForROM(Self[RandROM]);

    If Emu <> nil then
    begin
      //LaunchRom(Self[RandROM], Emu);
      Emu.RunROM(Self[RandROM]);
      //Save the ROM Data
      Self.SaveToFile();
    end
    else
      raise EJException.Create(QP_EMULATOR_NOT_FOUND);
  end;

end;

{-----------------------------------------------------------------------------}

Procedure TQPROMlist.SaveToFile();
var
  z : integer;
  StrList : TStrings;
  ROM : TQPRom;
begin

  //need to check if the RomFile parameter is blank or not
  if _FileName = '' then
    exit;

  //check that we are 'allowed' to write to the disk
  If not _AllowWrite then
    exit;

  StrList := TStringList.Create;
  Try

    StrList.Clear;
    StrList.Capacity := self.Count;

    For z := 0 to self.Count-1 do
    Begin

      Rom := self[z];
      If (Rom <> nil) and (ROM.name <> '') AND (ROM.path <>'') then
        StrList.Add(ROM.ToString);

    end;

    StrList.SaveToFile(_FileName);
  Finally
    FreeAndNil(StrList);
  end;
    
end;

{-----------------------------------------------------------------------------}

procedure TQPROMlist.SetMameCategories(SettingsDir : String);
Var
  I : integer;
  ini : TMemIniFile;
begin

  if fileexists(SettingsDir + 'catver.ini') then
  begin
    ini := TMemIniFile.Create(SettingsDir+'catver.ini');
    try

      for i := 0 to self.Count-1 do
        self[i].SetMameGameCategory(Ini);

      self.SaveToFile();

    finally
      FreeAndNil(ini);
    end;
  end
  else
    raise EJException.Create(QP_NO_CATVER);

end;

procedure TQPROMlist.SetMameMultiPlayer(SettingsDir : String);
Var
  I : integer;
  ini : TMemIniFile;
begin

  if fileexists(SettingsDir + 'nplayers.ini') then
  begin
    ini := TMemIniFile.Create(SettingsDir+'nplayers.ini');
    try

      for i := 0 to self.Count-1 do
        self[i].SetMameMultiplayer(Ini);

      self.SaveToFile();

    finally
      FreeAndNil(ini);
    end;
  end
  else
    raise EJException.Create(QP_NO_NPLAYERS);

end;


end.
