//------------------------------------------------------------------------
// UNIT information
//------------------------------------------------------------------------
{

This unit contains the Settings Class, which is used to store all of
Quickplays variables and settings, and has code to maintain these settings.
Also contains code to create default files such as Types and System

}

unit uSettings;

interface

uses SysUtils, Classes, Dialogs, forms, graphics, Inifiles, VirtualTrees;

Type
  TQPSettings = Class
    Private
      {Folder and File Locations}
      _AppDir,     //holds the path to the application directory
      _CfgDir, //holds the path to the DAT directory
      _ROMSdir,    //holds the path to the data directory (ROMS)
      _DirSearch  : String; //holds the path to search results directory.
      _QPlogo : TFileName; //holds the path to the QuickPlay logo.

      //variable to store whether QuickPlay can write to this disk
      _AllowWrite : Boolean;
      _NoDats : Boolean;
      _DatsUpToDate : Boolean;

      {Main Config Variables}
      _OptAutoDel,
      _OptMiniOnRun,
      _OptOldMini,
      _OptUseRunDLG,
      _OptTempDir,
      _OptClearFilter,
      _OptRememberROM,
      _OptSaveCols,
      _OptRomImages,
      _OptSearchROMDir,
      _OptUseParentsForCloneMedia,
      _OptIncVersion : boolean;
      _OptSearchType : Integer;

      // Variables for Dir treeview background.
      _OptUseDirBG : Boolean;
      _OptDirBGImage : TFileName;
      
      Function GetSettingsFile():TFileName;
      Function GetFavFile():TFileName;
      Function GetUncompressTo : String;

    Public

      Constructor Create;
      procedure BackupData(UseName : TFileName);
      Procedure CreateDefaultIni(Reset : boolean; Path : String);
      procedure CreateLanguage(path : string);
      procedure CreateSysDat(path: string);
      procedure CreateTypeDat(path : string);
      Procedure HideShowCol(Directory : String);
      procedure LoadIni();
      Procedure LoadLastROM();
      procedure RestoreData(DataFile : TFileName);
      procedure PathSetup();
      Procedure SaveAllSettings();
      procedure SaveCosmeticSettings();
      Procedure SaveCurrentROM(Dir : String; ROM : TObject);
      Procedure SetupWindow();

      // Paths Properties
      Property QPDirectory : String read _AppDir;
      Property SettingsDir : String Read _CfgDir;
      Property ROMsDir : String Read _ROMSdir;
      Property SearchDir : String read _DirSearch;
      Property FavsFile : TFileName Read GetFavFile;
      Property SettingsFile : TFileName Read GetSettingsFile;
      Property Logo : TFileName Read _QPLogo;
      Property UncompressTo : String read GetUncompressTo;

      //property to say whether file write access is allowed.
      Property AllowWrite : Boolean Read _AllowWrite write _AllowWrite;

      //General Properties
      Property AutoDelRoms : boolean Read _OptAutoDel Write _OptAutoDel;
      Property MiniOnRun : Boolean Read _OptMiniOnRun Write _OptMiniOnRun;
      Property UseOldMini : Boolean Read _OptOldMini Write _OptOldMini;
      Property UseRunDlg : Boolean Read _OptUseRunDLG Write _OptUseRunDLG;
      Property RememberLast : Boolean Read _OptRememberROM write _OptRememberROM;
      Property UseTempDir : Boolean Read _OptTempDir Write _OptTempDir;
      Property UseImagesForROMs : Boolean Read _OptRomImages Write _OptRomImages;
      Property ClearFilter : Boolean Read _OptClearFilter Write _OptClearFilter;

      Property SaveColumnSize : Boolean Read _OptSaveCols Write _OptSaveCols;
      Property SearchROMDir : Boolean Read _OptSearchROMDir Write _OptSearchROMDir;
      Property ParentsForClones : Boolean Read _OptUseParentsForCloneMedia Write _OptUseParentsForCloneMedia;
      Property AddVersion : Boolean Read _OptIncVersion Write _OptIncVersion;
      Property SearchType : Integer Read _OptSearchType Write _OptSearchType;

      Property NoDats : Boolean Read _NoDats Write _NoDats;
      Property DatsCurrent : Boolean Read _DatsUpToDate write _DatsUpToDate;

      //variables for Dir VT image
      Property UseBGonDir : Boolean Read _OptUseDirBG write _OptUseDirBG;
      Property DirBGImage :TFileName Read _OptDirBGImage Write _OptDirBGImage;

  end;

implementation

uses JCLFileUtils, JCLStrings, fMain, uJFile, uRom,
     JclSysInfo, uJUtilities, uJZip, uQPConst;

{$I Compilers.inc}

{-----------------------------------------------------------------------------}

Function TQPSettings.GetSettingsFile() : TFileName;
Begin
  Result := _CfgDir + 'settings.ini';
end;

Function TQPSettings.GetFavFile():TFileName;
Begin
  Result := _CfgDir + 'fav.dat';
end;

Function TQPSettings.GetUncompressTo : String;
begin
{  If _AllowWrite then
    UseDir := QPDirectory + 'temp\'
  else
    UseDir := ExtractFilePath(Zip);

  //now we will test if we can actually extract this file
  OldCurrent := GetCurrentDir();
  SetCurrentDir(UseDir);
  try
    If DiskFree(0) = 0 then
      UseDir := IncludeTrailingPathDelimiter(GetWindowsTempFolder()) + 'qp\temp\';
  finally
    SetCurrentDir(OldCurrent);
  end;    }
end;

{-----------------------------------------------------------------------------}

procedure TQPSettings.BackupData(UseName : TFileName);
var
  zFilter : TStrings;
begin
  zFilter := TStringList.Create;
  try
    zFilter.Add('data\*.*');
    zFilter.Add('dats\*.*');
    uJZip.ZipDirectory(_AppDir, UseName, zFilter);
  finally
    FreeAndNil(zFilter);
  end;
end;
{-----------------------------------------------------------------------------}

Procedure TQPSettings.CreateDefaultIni(Reset : boolean; Path : String);
Var
  Ini : TMemIniFile;
Begin

  If (Reset = true) OR (FileExists(Path) = false) then
  Begin
    Ini := TMemIniFile.Create(Path);

    try
    Ini.Clear;

    //General Settings

    Ini.WriteBool('Main','ChkAutoDel',False);
    Ini.WriteBool('Main','ChkMinimise', True);
    Ini.WriteBool('Main', 'ChkOldMini', false);
    Ini.WriteBool('Main', 'ChkRunDlg', False);
    Ini.WriteBool('Main', 'ChkClearFilter', true);
    Ini.WriteBool('Main', 'ChkRememberROM', True);
    Ini.WriteBool('Main', 'ChkTempDir', true);

    //ROM list options
    Ini.WriteBool('RomOptions','ChkCol',True);
    Ini.WriteBool('RomOptions','ChkRomGridlines',False);
    Ini.WriteBool('RomOptions','ChkRomHotTrack',False);
    Ini.WriteBool('RomOptions','ChkListImages',True);
    Ini.WriteBool('RomOptions','ChkRomRowSel',True);
    Ini.WriteBool('RomOptions','ChkRomHints', True);

    //Directory list options
    Ini.WriteBool('DirOptions', 'ChkDirGridlines', false);
    Ini.WriteBool('DirOptions', 'ChkDirHotTrack', false);
    Ini.WriteBool('DirOptions', 'ChkDirRowSel', false);
    Ini.WriteBool('DirOptions', 'ChkDirHints', true);

    //Media Options
    Ini.WriteBool('Main', 'ChkSearchROMdir', true);
    Ini.WriteBool('Main', 'ChkCloneSearch', true);
    Ini.WriteInteger('Main', 'SearchType', 1);

    //Mouse Options
    Ini.WriteBool('PadEmulation', 'ChkMouseEmu', False);
    Ini.WriteBool('PadEmulation', 'ChkPadAnalog', False);
    Ini.WriteInteger('PadEmulation', 'MouseCnt', 5);
    Ini.WriteInteger('PadEmulation', 'MouseRefresh', 10);

    //Version String
    Ini.WriteString('Version', 'QP', QPVer );
    if _AllowWrite then
      Ini.UpdateFile;

    //free the ini file object
    finally
      FreeAndNil(Ini);
    end;

  end;


End;

{-----------------------------------------------------------------------------}

procedure TQPSettings.CreateLanguage(path : string);
var
  lang : Tstringlist;
  tmpStream: TResourceStream;
begin

  if not _AllowWrite then
    Exit; 

  lang := Tstringlist.Create;
  try

    tmpStream := TResourceStream.Create( HInstance, 'language_txt', 'TEXT' );
    try
      lang.LoadFromStream( tmpStream );
    finally
      FreeAndNil(tmpStream);
    end;

    lang.SaveToFile(path);
  finally
    FreeAndNil(lang);
  end;
end;

{-----------------------------------------------------------------------------}
procedure TQPSettings.CreateSysDat(path : string);
var
  dat : Tstringlist;
  tmpStream: TResourceStream;
begin

  if not _AllowWrite then
    Exit;

  dat := Tstringlist.Create;
  try
    tmpStream := TResourceStream.Create( HInstance, 'systems_txt', 'TEXT' );
    try
      dat.LoadFromStream( tmpStream );
    finally
      FreeAndNil(tmpStream);
    end;
    dat.SaveToFile(path);
  finally
    FreeAndNil(dat);
  end;
end;

{-----------------------------------------------------------------------------}

procedure TQPSettings.CreateTypeDat(path : string);
var
  Types : Tstringlist;
  tmpStream: TResourceStream;
begin

  if not _AllowWrite then
    Exit;

  Types := Tstringlist.Create;
  try

    tmpStream := TResourceStream.Create( HInstance, 'type_txt', 'TEXT' );
    try
      Types.LoadFromStream( tmpStream );
    finally
      FreeAndNil(tmpStream);
    end;
  
    Types.SaveToFile(path);
  finally
    FreeAndNil(Types);
  end;
end;

{-----------------------------------------------------------------------------}

Procedure TQPSettings.HideShowCol(Directory : String);
Var
  I : Integer;
  Ini : TMemIniFile;
  ColSet : Boolean;

Begin

  ColSet := False;

  If Directory <> '' then
  begin

    Ini := TMemIniFile.Create(Directory + 'folders.ini');
    Try

     If Ini.SectionExists('ColView') then
     begin
       For i := 0 to MainFrm.VTRoms.Header.Columns.Count-1 do
         if Ini.ReadBool('ColView', IntToStr(i), true) then
           MainFrm.VTRoms.Header.Columns.Items[i].Options := MainFrm.VTRoms.Header.Columns.Items[i].Options + [coVisible]
         Else
           MainFrm.VTRoms.Header.Columns.Items[i].Options := MainFrm.VTRoms.Header.Columns.Items[i].Options - [coVisible];

       ColSet := True;
     end;

    Finally
      Ini.Free;
    end;
  end;

  If not ColSet then
  begin
    Ini := TMemIniFile.Create(SettingsFile);
    Try

     For i := 0 to MainFrm.VTRoms.Header.Columns.Count-1 do
       if Ini.ReadBool('ColView', IntToStr(i), true) then
         MainFrm.VTRoms.Header.Columns.Items[i].Options := MainFrm.VTRoms.Header.Columns.Items[i].Options + [coVisible]
       Else
         MainFrm.VTRoms.Header.Columns.Items[i].Options := MainFrm.VTRoms.Header.Columns.Items[i].Options - [coVisible];

    Finally
      Ini.Free;
    end;
  end;
  
End;

{-----------------------------------------------------------------------------}

//procedure to set up the window size and other related settings
procedure TQPSettings.LoadIni();
var
  Ini : TMemIniFile;
  I : Integer;
  AllPos: Boolean;
begin

  with MainFrm do
  begin

    Ini := TMemIniFile.Create(SettingsFile);
    try

      //check that all dat files are up to date.
      If StrCompare(Ini.ReadString('Version', 'QP', ''), '3.6') = 0 then
        _DatsUpToDate := False;

      MainFrm.SplitImagePos := Ini.ReadInteger('Windows', 'SplitImage', 185);
      MainFrm.SplitDirPos := Ini.ReadInteger('Windows', 'SplitDir', 177);


      // SET UP VARIABLES

      //Load the AutoDelete roms linked to deleted emulator option
      _OptAutoDel := Ini.ReadBool('Main','ChkAutoDel', False);

      //Load the Minimise when running a rom value
      _OptMiniOnRun := Ini.ReadBool('Main','ChkMinimise', True);

      //this variable allows you to use the OLD style minimising.
      _OptOldMini := Ini.ReadBool('Main', 'ChkOldMini', false);

      //load the use Run Rom Dialog when launching a game
      _OptUseRunDLG := Ini.ReadBool('Main', 'ChkRunDlg', True);

      //the use QP's temp dir for zip extraction setting.
      _OptTempDir := Ini.ReadBool('Main', 'ChkTempDir', true);

      //clear filter variable - boolean
      _OptClearFilter := Ini.ReadBool('Main', 'ChkClearFilter', true);

      //remember ROM variable
      _OptRememberROM := Ini.ReadBool('Main', 'ChkRememberROM', True);

      //load the media search ROM dir variable
      _OptSearchROMDir := Ini.ReadBool('Main', 'ChkSearchROMdir', true);

      //load the use parents name for Clones variable
      _OptUseParentsForCloneMedia := Ini.ReadBool('Main', 'ChkCloneSearch', true);

      //load the media search type variable
      _OptSearchType := Ini.ReadInteger('Main', 'SearchType', 1);

      {------------------------------------}
      // SET UP THE ROM LISTING NOW

      //Load the Save Column Sizes option
      _OptSaveCols := Ini.ReadBool('RomOptions','ChkCol', True);

      MainFrm.VTRoms.ShowHint := Ini.ReadBool('RomOptions', 'ChkRomHints', true);

      VTRoms.Header.SortColumn := Ini.ReadInteger('Windows', 'SortColumn', 0);
      If Ini.ReadBool('Windows', 'SortAsc', true) then
        VTRoms.Header.SortDirection := sdAscending
      else
        VTRoms.Header.SortDirection := sdDescending;

      //Load the Gridlines on VTRoms option.
      with MainFrm.VTRoms.TreeOptions do
      begin

        case (Ini.ReadBool('RomOptions','ChkRomRowSelect', True)) of
          false: SelectionOptions := (SelectionOptions - [toFullRowSelect]);
          true: SelectionOptions := (SelectionOptions + [toFullRowSelect]);
        end;

        case (Ini.ReadBool('RomOptions','chkRomGridlines', True)) of
          false: PaintOptions := (PaintOptions-[toShowHorzGridLines, toShowVertGridLines]);
          true: PaintOptions := (PaintOptions+[toShowHorzGridLines, toShowVertGridLines]);
        end;

        //Load the HotTrack on VTRoms option
        case (Ini.ReadBool('RomOptions','ChkRomHotTrack', False)) of
          false: PaintOptions := PaintOptions-[TohotTrack];
          true: PaintOptions := PaintOptions+[TohotTrack];
        end;

      end;  //end of with MainFrm.VTRoms.treeoptions code

      //Set the column positions.

      with MainFrm.VTRoms do
      Begin

      //check we have a list of column positions
      AllPos := true;
      For I := 0 to Header.Columns.Count-1 do
        If Ini.ValueExists('ColPos', IntToStr(I)) = false then
          AllPos := False;

      if AllPos then
        For I := 0 to Header.Columns.Count-1 do
           Header.Columns.Items[I].Position := Ini.ReadInteger('ColPos', IntToStr(i), 0);

      //If Column Save is turned on then load the sizes from Ini
      if _OptSaveCols then
        For i := 0 to Header.Columns.Count-1 do
          If Ini.ValueExists('ColSize', IntToStr(i)) then
            Header.Columns[i].Width := Ini.ReadInteger('ColSize', IntToStr(i), 50);

      //set up which columns to show and which to hide.
      HideShowCol(MainFrm.GetSelectedFolder);

      _OptIncVersion := Ini.ReadBool('RomOptions', 'ChkIncVersion', false);

      //Load the Images in listview option
      _OptRomImages := Ini.ReadBool('RomOptions','ChkListImages', True);

      // set the appearance options
      font.Name := ini.ReadString('RomOptions','FontName', 'Tahoma');
      font.Color := StringToColor(ini.Readstring('RomOptions','FontColour', 'ClWindowText'));
      font.Size := ini.ReadInteger('RomOptions','FontSize', 8);
      Font.Style := [];
      if ini.ReadBool('RomOptions','FontBold', false) = true then
        font.Style := font.Style + [fsbold];
      if ini.ReadBool('RomOptions','FontItalic', false) = true then
        font.Style := font.Style + [fsItalic];
      if ini.ReadBool('RomOptions','FontUnderline', false) = true then
        font.Style := font.Style + [fsUnderLine];
      if ini.ReadBool('RomOptions','FontStrike', false) = true then
        font.Style := font.Style + [fsstrikeout];

      Colors.HotColor := StringToColor(ini.Readstring('RomOptions','CboRTrack', 'clBlue'));
      Colors.FocusedSelectionColor := StringToColor(ini.Readstring('RomOptions','CboRSel', 'clSkyBlue'));
      Colors.FocusedSelectionBorderColor := StringToColor(ini.Readstring('RomOptions','CboRSel', 'clSkyBlue'));
      Colors.UnfocusedSelectionColor := StringToColor(ini.Readstring('RomOptions','CboRSelUnFoc', 'clSilver'));
      Colors.UnfocusedSelectionBorderColor := StringToColor(ini.Readstring('RomOptions','CboRSelUnFoc', 'clSilver'));
      Color := StringToColor(ini.Readstring('RomOptions','CboRBKCol', 'clWindow'));

      end; //end of With MainFrm.VTRoms

      // END OF SETTING UP ROM LISTING.

     {-------------------------------------}

      // SET UP THE DIRECTORY LISTING

      MainFrm.VTdir.ShowHint := Ini.ReadBool('DirOptions', 'ChkDirHints', true);
      
      with MainFrm.VTDir.TreeOptions do
      begin

        case (Ini.ReadBool('DirOptions','ChkDirRowSel', True)) of
          false: SelectionOptions := (SelectionOptions - [toFullRowSelect]);
          true: SelectionOptions := (SelectionOptions + [toFullRowSelect]);
        end;

        case (Ini.ReadBool('DirOptions','chkDirGridlines', True)) of
          false: PaintOptions := (PaintOptions-[toShowHorzGridLines, toShowVertGridLines]);
          true: PaintOptions := (PaintOptions+[toShowHorzGridLines, toShowVertGridLines]);
        end;

        //Load the HotTrack on VTRoms option
        case (Ini.ReadBool('DirOptions','ChkDirHotTrack', False)) of
          false: PaintOptions := PaintOptions-[TohotTrack];
          true: PaintOptions := PaintOptions+[TohotTrack];
        end;

      end;

      // set the appearance options
      with MainFrm.VTdir do
      begin
      font.Name := ini.ReadString('DirOptions','FontName', 'Tahoma');
      font.Color := StringToColor(ini.Readstring('DirOptions','FontColour', 'ClWindowText'));
      font.Size := ini.ReadInteger('DirOptions','FontSize', 8);
      Font.Style := [];
      if ini.ReadBool('DirOptions','FontBold', false) = true
        then font.Style := font.Style + [fsbold];
      if ini.ReadBool('DirOptions','FontItalic', false) = true
        then font.Style := font.Style + [fsItalic];
      if ini.ReadBool('DirOptions','FontUnderline', false) = true
        then font.Style := font.Style + [fsUnderLine];
      if ini.ReadBool('DirOptions','FontStrike', false) = true
        then font.Style := font.Style + [fsstrikeout];

      Colors.HotColor := StringToColor(ini.Readstring('DirOptions','CboDTrack', 'clBackground'));
      Colors.FocusedSelectionColor := StringToColor(ini.Readstring('DirOptions','CboDSel', 'clSkyBlue'));
      Colors.UnfocusedSelectionColor := StringToColor(ini.Readstring('DirOptions','CboDSelUnFoc', 'clSilver'));
      Color := StringToColor(ini.Readstring('DirOptions','CboDBKCol', 'clWindow'));

      end; //end of with MainFrm.vtdir do

      //set up the image bkground
      _OptUseDirBG := Ini.ReadBool('DirOptions', 'UseDirBG', False);
      _OptDirBGImage := Ini.ReadString('DirOptions', 'DirBGImage', '');
      
      // END OF SETTING UP DIRECTORY LISTING.

      {------------------------------------}

      // SET UP THE MISC APPEARANCE OPTIONS
      
      // set up visiblility of the Directory Toolbar
      If (not Ini.ReadBool('Windows', 'ShowDBar', true)) and
         (MainFrm.TBDir.Visible) then
        ActDBarVisible.Execute;

      //set up the visibility of the ROM toolbar
      If (not Ini.ReadBool('Windows', 'ShowRBar', true)) and
         (MainFrm.TBRoms.Visible) then
        ActRBarVisible.Execute;

      //set up the visibility of the ROM toolbar
      If (not Ini.ReadBool('Windows', 'ShowFilter', true)) and
         (MainFrm.PanFilter.Visible) then
        ActFilterVisible.Execute;

      //set up the rom image panel
      If (not Ini.ReadBool('Windows', 'ShowImg', True)) and
         (MainFrm.PanImage.Visible) then
        ActImgVisible.Execute;

    finally
      FreeAndNil(Ini);
    end;

  end; //end of WITH MainFrm statement

end;

{-----------------------------------------------------------------------------}

Procedure TQPSettings.LoadLastROM();
var
  Ini : TMemIniFile;
  Dir, ROM : String;
  SType : TQPRefresh;
begin

  If _OptRememberROM then
  begin
    Ini := TMemIniFile.Create(SettingsFile);
    try
      Dir := Ini.ReadString('LastROM', 'Dir', '');
      ROM := Ini.ReadString('LastROM', 'ROM', '');
      SType := TQPRefresh(Ini.ReadInteger('LastROM', 'Type', 1));
      MainFrm.JumpToROM(Dir, ROM, SType);
    Finally
      FreeAndNil(Ini);
    end;
  end;

end;

{-----------------------------------------------------------------------------}

procedure TQPSettings.PathSetup;
var
  ReadOnly : Boolean;
begin

  ReadOnly := ( DiskFree(0) = 0 );

  // The QuickPlay executable directory
  _AppDir := extractfilepath(paramstr(0));

  //  The directories for the various DAT files which QuickPlay requires
  _CfgDir := _AppDir + 'dats\';

  //The Data directories, which contain the roms' databases
  _ROMSdir := _AppDir + 'data\';

  _DirSearch := _AppDir + 'search\';

  //now check that both these directories exist AND are writeable!
  if (ReadOnly) and
     (DirectoryExists(_CfgDir) = false) and
     (DirectoryExists(_ROMSdir) = false)  then
  begin
    _AppDir := IncludeTrailingPathDelimiter(GetWindowsTempFolder) + 'qp\';
    CreateDir(_AppDir);
    //now CHANGE the 2 directory variables.
    _CfgDir := _AppDir + 'dats\';
    _ROMSdir := _AppDir + 'data\';
    _DirSearch := _AppDir + 'search\';
    //we need to move the efind files so they are writeable
    if not DirectoryExists(_AppDir + 'efind\') then
      DirCopy(extractfilepath(paramstr(0))+'efind\', _AppDir + 'efind\');
  end;

  _AllowWrite := (not IsDriveReadOnly(_AppDir));

  If (DirectoryExists(_ROMSdir) = false) and (DirectoryExists(_CfgDir) = false) then
    _NoDats := True
  else
    _NoDats := False;

  //If the DATs dir doesnt exist, it is created.
  if not directoryExists(_CfgDir) then
    CreateDir(_CfgDir);

  if not directoryExists(_DirSearch) then
    CreateDir(_DirSearch);

  //Check is the settings.ini file exists, if it doesnt - create it
  if not fileexists(SettingsFile) then
    CreateDefaultIni(false, SettingsFile);

  //Check is the system.dat file exists, if it doesnt - create it
  if not fileexists(_CfgDir + 'systems.dat') then
    CreateSysDat(_CfgDir+'systems.dat');

  //check the language file exists, and if not create it
  if not FileExists(_CfgDir + 'language.txt') then
    CreateLanguage(_CfgDir + 'language.txt');

  //check the game type dat file exists, and if not create it
  if not FileExists(_CfgDir + 'type.txt') then
    CreateTypeDat(_CfgDir + 'type.txt');

  //make sure the Data directory exists.
  if not DirectoryExists(_ROMSdir) then
  begin
    If not _NoDats then
      MessageDlg(QP_ROMS_DATA_DIR_NOT_FOUND, mtInformation, [mbOK], 0);
    if not CreateDir(_ROMSdir) then //create the directory specified.
      raise Exception.Create(QP_CREATE_DIR_FAILED);
    if not CreateDir(_ROMSdir+'quickplay') then //create the directory specified.
      raise Exception.Create(QP_CREATE_DIR_FAILED);
  end;

  //if the logo file exists, set the variable to a non null value.
  If FileExists(_AppDir + 'qp.gif') then
    _QPLogo := _AppDir + 'qp.gif'
  else
    _QPlogo := '';

  //if the folders.ini file exists in the DAT dir then delete it
  If FileExists(_CfgDir + 'folders.ini') then
    DeleteFile(_CfgDir + 'folders.ini');

end;

{-----------------------------------------------------------------------------}

constructor TQPSettings.Create;
begin

  _DatsUpToDate := True;

  PathSetup;

  If not _AllowWrite then
    MessageDlg(QP_DRIVE_READ_ONLY, mtWarning, [mbOK], 0);

  //set up the window
  SetupWindow();

  //Set up the window size and other settings from the registry
  LoadIni();
end;

{-----------------------------------------------------------------------------}

procedure TQPSettings.RestoreData(DataFile : TFileName);
begin

  //if we cant write to disk there's no point going on
  If _AllowWrite = false then
    raise EJWarning.Create(QP_DRIVE_READ_ONLY_SHORT);

  //first we delete the QP Data directories.
  DelTree(_ROMSdir);
  DelTree(_CfgDir);
  uJZip.UnzipZip(DataFile, _AppDir, False, True, True);

end;
{-----------------------------------------------------------------------------}

Procedure TQPSettings.SaveAllSettings();
var
  Ini : TMemIniFile;
begin
  Ini := TMemIniFile.create(MainFrm.Settings.SettingsFile);
  try

    Ini.EraseSection('Main');

    //General Options
    Ini.WriteBool('Main', 'ChkAutoDel', _OptAutoDel);
    Ini.WriteBool('Main', 'ChkMinimise', _OptMiniOnRun);
    Ini.WriteBool('Main', 'ChkOldMini', _OptOldMini);
    Ini.WriteBool('Main', 'ChkRunDlg', _OptUseRunDLG);
    Ini.WriteBool('Main', 'ChkClearFilter', _OptClearFilter);
    Ini.WriteBool('Main', 'ChkRememberROM', _OptRememberROM);
    Ini.WriteBool('Main', 'ChkTempDir', _OptTempDir);

    //Media Path options
    Ini.WriteBool('Main', 'ChkSearchROMdir', _OptSearchROMDir);
    Ini.WriteBool('Main', 'ChkCloneSearch', _OptUseParentsForCloneMedia);
    Ini.WriteInteger('Main', 'SearchType', _OptSearchType);

    //ROM list options
    Ini.WriteBool('RomOptions', 'ChkRomHints', MainFrm.VTRoms.ShowHint);
    Ini.WriteBool('RomOptions', 'chkIncVersion', _OptIncVersion);
    Ini.WriteBool('RomOptions', 'ChkCol', _OptSaveCols);
    Ini.WriteBool('RomOptions', 'ChkListImages', _OptRomImages);

    with MainFrm.VTRoms.TreeOptions do
    begin
      Ini.WriteBool('RomOptions', 'ChkRomRowSelect', (toFullRowSelect in SelectionOptions) );
      Ini.WriteBool('RomOptions', 'chkROMGridlines', (toShowHorzGridLines in PaintOptions) );
      Ini.WriteBool('RomOptions', 'ChkROMHotTrack', (TohotTrack in PaintOptions) );
    end;
    With MainFrm.VTRoms do
    begin
      //Rom List Colours
      ini.WriteString('RomOptions', 'CboRBKCol', ColorToString(Color));
      ini.WriteString('RomOptions', 'CboRSel', ColorToString(Colors.FocusedSelectionColor));
      ini.WriteString('RomOptions', 'CboRSelUnFoc', ColorToString(Colors.UnfocusedSelectionColor));
      ini.WriteString('RomOptions', 'CboRTrack', ColorToString(Colors.HotColor));

      ini.WriteString('RomOptions','FontName', font.Name);
      ini.WriteString('RomOptions','FontColour', ColorToString(Font.Color));
      ini.writeInteger('RomOptions','FontSize', Font.Size);

      ini.WriteBool('RomOptions', 'FontBold', (fsbold in font.Style));
      ini.WriteBool('RomOptions', 'FontItalic', (fsItalic in font.Style));
      ini.WriteBool('RomOptions', 'FontUnderline', (fsUnderLine in font.Style));
      ini.WriteBool('RomOptions', 'FontStrike', (fsStrikeOut in font.Style));

    end;

    with MainFrm.VTdir.TreeOptions do
    begin
      // Directory List Options
      Ini.WriteBool('DirOptions', 'chkDirRowSel', (toFullRowSelect in SelectionOptions) );
      Ini.WriteBool('DirOptions', 'chkDirGridlines', (toShowHorzGridLines in PaintOptions) );
      Ini.WriteBool('DirOptions', 'ChkDirHotTrack', (TohotTrack in PaintOptions) );
      Ini.WriteBool('DirOptions', 'ChkDirHints', MainFrm.VTDir.ShowHint );
    end;

    Ini.WriteBool('DirOptions', 'UseDirBG', Self._OptUseDirBG);
    Ini.WriteString('DirOptions', 'DirBGImage', Self._OptDirBGImage);

    With MainFrm.VTdir do
    begin

      //Rom List Colours
      ini.WriteString('DirOptions', 'CboDBKCol', ColorToString(Color));
      ini.WriteString('DirOptions', 'CboDSel', ColorToString(Colors.FocusedSelectionColor));
      ini.WriteString('DirOptions', 'CboDSelUnFoc', ColorToString(Colors.UnfocusedSelectionColor));
      ini.WriteString('DirOptions', 'CboDTrack', ColorToString(Colors.HotColor));

      ini.WriteString('DirOptions','FontName', Font.Name);
      ini.WriteString('DirOptions','FontColour', ColorToString(Font.Color));
      ini.writeInteger('DirOptions','FontSize', Font.Size);

      ini.WriteBool('DirOptions', 'FontBold', (fsBold in font.Style));
      ini.WriteBool('DirOptions', 'FontItalic', (fsItalic in font.Style));
      ini.WriteBool('DirOptions', 'FontUnderline', (fsUnderLine in font.Style));
      ini.WriteBool('DirOptions', 'FontStrike', (fsStrikeOut in font.Style));

    end;

    // finally update the ini file on disk to reflect our changes.
    if MainFrm.Settings.AllowWrite then
      Ini.UpdateFile;
      
  finally
    FreeAndNil(Ini);
  end;
end;

{-----------------------------------------------------------------------------}

procedure TQPSettings.SaveCosmeticSettings;
Var
  Ini : TMemIniFile;
  I : Integer;
begin

  //check that we are 'allowed' to write to the disk
  If not _AllowWrite then
    exit;

  //Create the IniFile object
  Ini := TMemIniFile.Create(SettingsFile);
  try

    //write the quickplay version to the ini file.
    Ini.WriteString('Version', 'QP', QPVer );

    With MainFrm do
    Begin

    //Write the window size.
    if WindowState <> wsMaximized then
    begin
      Ini.WriteInteger('Windows','Top', Top);
      Ini.WriteInteger('Windows','Left', Left);
      Ini.WriteInteger('Windows','Width', Width);
      Ini.WriteInteger('Windows','Height', Height);
      Ini.WriteBool('Windows', 'IsMaximised', false);
    end
    else
      Ini.WriteBool('Windows', 'IsMaximised', true);


    Ini.WriteInteger('Windows', 'SplitDir', MainFrm.SplitDirPos);
    Ini.WriteInteger('Windows', 'SplitImage', MainFrm.SplitImagePos);

    //Save the column widths.
    if _OptSaveCols = true then
    Begin
      Ini.EraseSection('ColSize');
      For i := 0 to VTRoms.Header.Columns.Count-1 do
        Ini.WriteInteger('ColSize', IntToStr(i), VTRoms.Header.Columns[i].Width);
    end;

    //save the currently SORTED column value and its direction
    Ini.WriteInteger('Windows', 'SortColumn', VTRoms.Header.SortColumn);
    If VTRoms.Header.SortDirection = sdAscending then
      Ini.WriteBool('Windows', 'SortAsc', true)
    else
      Ini.WriteBool('Windows', 'SortAsc', false);

    //save the column positions
    Ini.EraseSection('ColPos');
    For I := 0 to VTRoms.Header.Columns.Count-1 do
    Begin
      Ini.WriteInteger('ColPos', IntToStr(I), VTRoms.Header.Columns.Items[i].Position);
    end;

    //save the status of the image panel
    Ini.WriteBool('Windows', 'ShowImg', PanImage.Visible);

    //save whether the ROM toolbar is visible
    Ini.WriteBool('Windows', 'ShowRBar', TBRoms.Visible);

    //save whether the Dir toolbar is visible
    Ini.WriteBool('Windows', 'ShowDBar', TBDir.Visible);

    //save whether the filter txtbox is visible
    Ini.WriteBool('Windows', 'ShowFilter', PanFilter.Visible);

    Ini.UpdateFile;

    end; //end of WITH MainFrm

  finally
    FreeAndNil(ini);
  end;
end;

{-----------------------------------------------------------------------------}

Procedure TQPSettings.SaveCurrentROM(Dir : String; ROM : TObject);
var
  Ini : TMemIniFile;
begin

  //check that we are 'allowed' to write to the disk
  If not _AllowWrite then
    exit;

  Ini := TMemIniFile.Create(SettingsFile);
  Try

    If Ini.ReadBool('Main', 'ChkRememberROM', false) then
    begin

      //save the current selected directory
      Ini.WriteString('LastROM', 'Dir', Dir);

      If ROM = nil then
      begin
        Ini.WriteString('LastROM', 'ROM', '');
        Ini.WriteInteger('LastROM', 'Type', 1);
      end
      else If TQPROM(ROM).ZipName <> '' then
      begin
        Ini.WriteString('LastROM', 'ROM', TQPROM(ROM).ZipName);
        Ini.WriteInteger('LastROM', 'Type', 3);
      end
      else if TQPROM(ROM).CloneName <> '' then
      begin
        Ini.WriteString('LastROM', 'ROM', TQPROM(ROM).CloneName);
        Ini.WriteInteger('LastROM', 'Type', 0);
      end
      else
      begin
        Ini.WriteString('LastROM', 'ROM', TQPROM(ROM).Path);
        Ini.WriteInteger('LastROM', 'Type', 2);
      end;

      
    end //end of Save Last ROM setting.
    else
      If Ini.SectionExists('LastROM') then
        Ini.EraseSection('LastROM');

    Ini.UpdateFile;
    
  Finally
    FreeAndNil(Ini);
  end;

end;
{-----------------------------------------------------------------------------}

Procedure TQPSettings.SetupWindow();
var
  Ini : TMemIniFile;
  Top, Left, Width, Height : Integer;
begin

   Ini := TMemIniFile.Create(SettingsFile);
   try
     //Set the window size to values stored in Ini File
     Top := Ini.ReadInteger('Windows','Top', 130);
     Left := Ini.ReadInteger('Windows', 'Left', 200);
     Width := Ini.ReadInteger('Windows', 'Width', 800);
     Height := Ini.ReadInteger('Windows', 'Height', 450);
     MainFrm.SetBounds(Left, Top, Width, Height);

     If Ini.ReadBool('Windows', 'IsMaximised', false) then
       MainFrm.WindowState := wsMaximized;

   finally
     FreeAndNil(Ini);
   end;
end;

{-----------------------------------------------------------------------------}

end.
